# -*- coding:utf-8 -*-
# @time:2023/3/139:28
# @author:LX
# @file:toolList.py
# @software:PyCharm
from PyQtGuiLib.header import (
    is_mac_sys,
    QWidget,
    QVBoxLayout,
    QPushButton,
    Signal,
    QObject,
    QSpacerItem,
    QSizePolicy,
    qt
)

from PyQtGuiLib.styles import QssStyleAnalysis


class ToolListItem(QVBoxLayout):
    clicked = Signal(QObject)

    def __init__(self,text:str=None,*args,**kwargs):
        super().__init__(*args,**kwargs)

        self.setSpacing(0)
        self.setContentsMargins(0,0,0,0)
        self.__item_btn = QPushButton()
        self.__item_btn.setObjectName("itemBtn")
        self.__item_widget = QWidget()
        self.__item_widget.hide()
        self.__item_widget.setObjectName("itemWidget")

        self.__item_btn.clicked.connect(self.btn_event)

        self.addWidget(self.__item_btn)
        self.addWidget(self.__item_widget)

        # qss
        self.__btnQss = QssStyleAnalysis(self.__item_btn)
        self.__widgetQss = QssStyleAnalysis(self.__item_widget)


        if text:
            self.setText(text)

        self.defaultStyle()

    def btn_event(self):
        if self.itemWidget().isHidden():
            self.itemWidget().show()
        else:
            self.itemWidget().hide()
        self.clicked.emit(self)

    def itemBtn(self) -> QPushButton:
        return self.__item_btn

    def itemWidget(self) -> QWidget:
        return self.__item_widget

    def setText(self,text:str):
        self.itemBtn().setText(text)

    def text(self) -> str:
        return self.__item_btn.text()

    def defaultStyle(self):
        self.__btnQss.setQSS('''
            #itemBtn{
                background-color: rgb(12, 11, 15);
                color: rgb(240, 240, 240);
                font: 12pt "华文楷体";
            }
            #itemBtn:hover{
                font: 13pt "华文楷体";
                color: rgb(255, 255, 127);
            }
                ''')

        self.__widgetQss.setQSS('''
        #itemWidget{
        border:1px solid rgb(12, 11, 15);
        }
        ''')

        if is_mac_sys:
            self.__btnQss.selector("#itemBtn").updateAttr("font",'14pt "Heiti SC"')
            self.__btnQss.selector("#itemBtn:hover").updateAttr("font",'16pt "Heiti SC"')

    def clear(self):
        self.itemBtn().disconnect()
        self.removeWidget(self.itemBtn())
        self.removeWidget(self.itemWidget())
        self.itemBtn().deleteLater()
        self.itemWidget().deleteLater()


class ToolListWidget(QWidget):
    clickItem = Signal(QObject)

    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)

        # 样式表生效
        self.setAttribute(qt.WA_StyledBackground, True)

        self.resize(250,350)
        self.setObjectName("ToolListWidget")
        self.__core_vboy = QVBoxLayout(self)
        self.__core_vboy.setSpacing(0)
        self.__core_vboy.setContentsMargins(0,0,0,0)

        self.verSpacer = QSpacerItem(0, 0, QSizePolicy.Minimum, QSizePolicy.Expanding)

        # Record all the ToolListItem
        self.__tool_items = []

        self.myEvent()
        self.defaultStyle()

    def __verSpacer(self):
        self.__core_vboy.removeItem(self.verSpacer)
        self.__core_vboy.addSpacerItem(self.verSpacer)

    def addItem(self,item:ToolListItem):
        item.clicked.connect(self.clickItem.emit)
        self.__core_vboy.addLayout(item)
        self.__tool_items.append(item)
        self.__verSpacer()

    def addItems(self,labels:list):
        for text in labels:
            item = ToolListItem(text)
            item.clicked.connect(self.clickItem.emit)
            self.__core_vboy.addLayout(item)
            self.__tool_items.append(item)
            self.__verSpacer()

    def indexItem(self,index) -> ToolListItem:
        return self.__tool_items[index]

    def removeItem(self,item:ToolListItem):
        if item in self.__tool_items:
            temp_item = self.__tool_items.pop(self.__tool_items.index(item))  # type:ToolListItem
            self.__core_vboy.removeItem(temp_item)
            temp_item.clear()

    def count(self) -> int:
        return len(self.__tool_items)

    def __top_event(self,item:ToolListItem):
        '''
            当所有节点收起时,添加一个弹簧,
            如果有一个节点是展开的,就删除弹簧
        '''
        temp = []
        for item in self.__tool_items:
            temp.append(item.itemWidget().isHidden())

        if temp.count(True) == len(self.__tool_items):
            self.__core_vboy.addSpacerItem(self.verSpacer)
        else:
            self.__core_vboy.removeItem(self.verSpacer)

    def myEvent(self):
        # Implement an event internally
        self.clickItem.connect(self.__top_event)

    def defaultStyle(self):
        self.setStyleSheet('''
        ToolListWidget{
        background-color: gray;
        }
        ''')

    def allItems(self)->[ToolListItem]:
        return self.__tool_items

    def getFoldItems(self)->[ToolListItem]:
        temp = []
        for item in self.allItems():
            if item.itemWidget().isHidden():
                temp.append(item)
        return temp

    def getUnfoldItems(self)->[ToolListItem]:
        temp = []
        for item in self.allItems():
            if not item.itemWidget().isHidden():
                temp.append(item)
        return temp


# -*- coding:utf-8 -*-
# @time:2023/2/1310:31
# @author:LX
# @file:eg5.py
# @software:PyCharm
from PyQtGuiLib.header import (
    PYQT_VERSIONS,
    QApplication,
    sys,
    QWidget,
    qt,
    QPushButton,
    QLabel,
    Qt,
)

from PyQtGuiLib.styles import QssStyleAnalysis


class Test(QWidget):
    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)
        self.resize(600,600)


        self.setAttribute(Qt.WA_StyledBackground,True)

        self.l = QLabel("测试标签",self)
        self.btn = QPushButton("测试按钮",self)
        self.btn.setObjectName("btn")
        self.btn2 = QPushButton("测试按钮2号",self)
        self.l.move(30, 30)
        self.btn.resize(150, 80)
        self.btn.move(80, 80)
        self.btn2.resize(100,60)
        self.btn2.move(250,80)

        '''
            解析 测试1 
        '''

        # 创建一个针对 测试按钮 的QSS解析器
        self.qss = QssStyleAnalysis(self.btn)
        # 对窗口上所有按钮,标签设置样式
        self.qss.setQSS('''
        QWidget{
        color: rgb(0, 255, 127);
        background-color:rgb(0, 0, 0);
        }
        QPushButton{
        color: rgb(0, 0, 0);
        background-color:rgb(255, 255, 255);
        }
        ''')

        # 重新创建一个针对 测试按钮 的QSS解析器
        # 通过 inherit() 可以将'测试按钮'的样式 传承到 '测试按钮2号' 上
        self.btnqss = QssStyleAnalysis(self.btn)
        self.btnqss.inherit()
        self.btnqss.setParent(self.btn2) # 关键语句

        # print(self.btnqss.toDict())
        # self.btnqss.selector("PushButton")
        self.btnqss.selector("QPushButton").updateAttr("color", "blue")
        print(self.btnqss.isSelectKey("QPushButton"))
        print(self.btnqss.selector("QPushButton").isAttr("color"))

        # 添加一个已经存在的属性会覆盖原来的,已视为BUG(已修复)
        # self.btnqss.appendQSS('''
        # QPushButton{
        #
        # }
        # ''')
        self.btnqss.appendQSSDict({
            "QPushButton": {
                "border": "1px solid blue"
            },
        })
        print("=====")
        print(self.btnqss.toStr())


if __name__ == '__main__':
    app = QApplication(sys.argv)

    win = Test()
    win.show()

    if PYQT_VERSIONS in ["PyQt6","PySide6"]:
        sys.exit(app.exec())
    else:
        sys.exit(app.exec_())


# -*- coding:utf-8 -*-
# @time:2023/6/518:07
# @author:LX
# @file:message.py
# @software:PyCharm
from PyQtGuiLib.header import (
    QWidget,
    QFrame,
    QPushButton,
    textSize,
    QFont,
    Qt,
    QLineEdit,
    QLabel,
    QIcon,
    QSize,
    QPixmap
)
import re
import time

from PyQtGuiLib.core.charRooms.headImage import HeadImage

from PyQtGuiLib.core.charRooms.utility import imageToPix,Head_Type

Left = "left"
Right = "right"

Type_Text = "text"
Type_Image = "image"


class Message:
    def __init__(self, name:str, data:Head_Type, type="text"):
        '''
        {
            "data":xxxx
            "name":xxx
            "type":xxxx
        }

        :param data:
        '''
        if data.endswith(".png") or data.endswith(".jpg") or data.endswith(".jpeg") or data.endswith(".heic") or data.endswith("webp"):
            type = "image"

        self.__data = {
            "name": name,
            "data": data,
            "type": type,
            "send_time":time.strftime("%Y-%m-%d %H:%M:%S",time.localtime()),
            "head_image": {
                "path": "",
            }
        }

        self.Max_Line_Num = 50  # 单行最大文本数量

        self.Max_Message_Num = 2000  # 单条消息的最大字数

        self.bgTemplate = None

    def isText(self) -> bool:
        return True if self.dataType() == "text" else False

    def isImage(self) -> bool:
        return True if self.dataType() == "image" else False

    def setHeadImage(self, path: str):
        self.__data["head_image"]["path"] = path

    def headImage(self) -> dict:
        return self.__data["head_image"]

    def isHeadImage(self) -> bool:
        return True if self.headImage()["path"] else False

    def getMaxMesNum(self) -> int:
        return self.Max_Message_Num

    def name(self) -> str:
        return self.__data["name"] + " {}".format(time.strftime("%H:%M:%S", time.localtime()))

    def dataType(self) -> str:
        return self.__data["type"]

    def data(self):
        return self.__data["data"]

    def length(self) -> int:
        return len(self.data())

    def setBGTemplate(self, obj):
        self.bgTemplate = obj

    def __getMaxStr(self, text_list) -> str:
        max_len = 0
        data = ""
        for text in text_list:
            n = len(text)
            if n > max_len:
                max_len = n
                data = text
        return data

    # 单行文本对折
    def strFoldHalf(self, str_text) -> str:
        if len(str_text) < self.Max_Line_Num:
            return None
        else:
            temp_texts = []
            s, e = 0, self.Max_Line_Num
            while True:
                data = str_text[s:e]
                if data:
                    temp_texts.append(data)
                else:
                    return temp_texts
                s = e
                e += self.Max_Line_Num

    def analysisText(self):
        if self.isText():
            text = self.data()
            newline_character_number = 1

            period_text = text.split("\n")

            # ---
            '''
                对文本折行,
                如果文本只有一句话,则单行最大文字数量为 60
                如果是多行,
                    在多行中,根据文章和的语言进行区分 单行最大文字数量
                    中文 > 英文 单行最大文字数量为 30
                  反之:
                    单行最大文字数量为 60
    
            '''
            chinese_data = re.findall(r"[\u4e00-\u9fa5]+", text)
            en_data = re.findall(r"[a-z]+", text, re.I)
            if chinese_data:
                chinese_num = len("".join(chinese_data))
            else:
                chinese_num = 0
            if en_data:
                en_num = len("".join(en_data))
            else:
                en_num = 0

            if len(period_text) == 1:
                if chinese_num > en_num:
                    self.Max_Line_Num = 30
                else:
                    self.Max_Line_Num = 60
                re_text = self.strFoldHalf(text)
                if re_text:
                    period_text.clear()
                    period_text.extend(re_text)
                    self.__data["data"] = "\n".join(period_text)
            else:
                if chinese_num > en_num:
                    self.Max_Line_Num = 30
                else:
                    self.Max_Line_Num = 50

                new_period_text = []
                for nt in period_text:
                    re_text = self.strFoldHalf(nt)
                    if not re_text:
                        new_period_text.append(nt)
                    else:
                        new_period_text.extend(re_text)
                self.__data["data"] = "\n".join(new_period_text)
                period_text = new_period_text
            # ===
            if "\n" in self.data():
                newline_character_number = len(period_text) + 1

            max_period_text = self.__getMaxStr(period_text)

            f = QFont(max_period_text)
            f.setPointSize(12)
            fsize = textSize(f, max_period_text)

            w, h = fsize.width() + 13, fsize.height() * newline_character_number + 5
            if h < 30:
                h = 30
        elif self.isImage():
            w, h = 300, 100

        self.__data["BGTemplate_Height"] = h + self.bgTemplate.profile_btn.height() + self.bgTemplate.profile_btn.pos().y()
        self.__data["MessageBody_Size"] = w, h

    def bgTemplateHeight(self) -> int:
        return self.__data["BGTemplate_Height"]

    def messageBodySize(self) -> tuple:
        return self.__data["MessageBody_Size"]


# 消息内容主体
class MessageBody(QLabel):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.mes = None  # type:# Message
        self.setAlignment(Qt.AlignLeft | Qt.AlignTop)

        self.setOpenExternalLinks(True)
        self.setTextInteractionFlags(Qt.LinksAccessibleByMouse | Qt.TextSelectableByMouse)

        f = QFont()
        f.setPointSize(12)
        self.setFont(f)

        self.setStyleSheet('''
        border:1px solid rgb(184, 184, 184);
        border-radius:5px;
        font: 12pt "等线";
        background-color: rgb(255, 170, 127);
        padding-left:5;
        padding-top:5;
                ''')

    def setMes(self, mes: Message):
        data = mes.data()
        if mes.isText():
            if re.findall("^http.*|^http.* ", data):
                self.setText("<a href='{}'>{}</a>".format(text, data))
            else:
                self.setText("{}".format(data))
        elif mes.isImage():
            self.setPixmap(imageToPix(data,QSize(100,100)))


# 单条消息模板
class BGTemplate(QFrame):
    def __init__(self, parent, direction, mes: Message, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.parent = parent

        self.setStyleSheet("background-color:#fff;")

        self.profile_btn = HeadImage(self)
        self.profile_btn.setFixedSize(50, 50)
        if mes.isHeadImage():
            self.profile_btn.setHeadImage(mes.headImage()["path"])

        mes.setBGTemplate(self)
        mes.analysisText()
        self.setFixedHeight(mes.bgTemplateHeight())

        #
        self.mes = MessageBody(self)
        self.mes.setMes(mes)
        self.direction = direction

        if direction == Left:
            self.moveLeft()
            self.name = QLabel(self)
            self.name.setText(mes.name())
            self.name.setStyleSheet('font: 25 10pt "等线 Light";')
            self.name.move(self.profile_btn.width() + 15, self.profile_btn.height() // 2 - 10)
        elif direction == Right:
            self.mes.show()
            self.moveRight()

        self.defaultStyle()

    def moveLeft(self):
        self.profile_btn.move(0, 10)
        self.mes.move(self.profile_btn.width() + 15, self.profile_btn.height() // 2 + 10)

    def moveRight(self):
        self.profile_btn.move(self.parent.width() - self.profile_btn.width() - 20, 10)
        self.mes.move(self.width() - self.mes.width() - self.profile_btn.width() - 25,
                      self.profile_btn.height() // 2 + 10)

    def defaultStyle(self):
        self.profile_btn.setStyleSheet('''
border:1px solid rgb(184, 184, 184);
border-radius:25px;
        ''')

    def resizeEvent(self, e) -> None:
        if self.direction == Right:
            self.moveRight()
        super().resizeEvent(e)


from PyQtGuiLib.header import (
    QWidget,
    QPainter,
    QPaintEvent,
    QPolygonF,
    QFont,
    QColor,
    QPointF,
    QBrush,
    textSize,
    QPainterPath,
    QRect,
    QPen,
    qt,
    pyqtProperty
)
from PyQtGuiLib.core.widgets import WidgetABC

class BubbleWidget(WidgetABC):

    # 方向
    Top = "top"
    Down = "Down"
    Left = "Left"
    Right = "Right"

    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)
        self.resize(100,60)

        # 箭头高度(三角形)
        self._arrows_h = 16

        # 文字
        self._text = ""

        # 追踪的控件
        self.trackWidget = None #type:QWidget

        self.__direction = BubbleWidget.Down

        #
        self.setText("气泡窗口")

    def set_Arrows(self,size:int):
        self._arrows_h = size

    def get_Arrows(self) -> int:
        return self._arrows_h
    # ----------------

    # 设置文字
    def setText(self,text:str):
        self._text = text
        self.textExtend()

    def text(self)->str:
        return self._text

    # 文字扩展
    def textExtend(self):
        f = QFont()
        f.setPointSize(self.get_fontSize())
        # 文字大小
        fs = textSize(f, self._text)
        fw = fs.width()
        fh = fs.height()

        if self.__direction in [BubbleWidget.Top,BubbleWidget.Down]:
            self.resize(fw + 30, self.height())
        else:
            self.resize(fw + 30+self._arrows_h, self.height())

        if fh >= self.height():
            self.resize(self.width(),self.height()+40)

        self.setTrack(self.trackWidget)

    # 设置气泡箭头方向
    def setDirection(self,d):
        self.__direction = d

    def direction(self)->str:
        return self.__direction

    # 控件追踪
    def setTrack(self, widget: QWidget):
        if widget is None:
            return

        self.trackWidget = widget

        x,y = widget.x(),widget.y()
        w,h = widget.width(),widget.height()
        cx = widget.x()+w//2-self.width()//2  # x轴中心
        cy = widget.y()+h//2-self.height()//2

        if self.__direction == BubbleWidget.Top:
            self.move(cx,y+h)
        elif self.__direction == BubbleWidget.Left:
            self.move(x+w,cy)
        elif self.__direction == BubbleWidget.Right:
            self.move(x-self.width(),cy)
        else:
            self.move(cx,y-self.height())

    # 绘制气泡
    def drawBubble(self,painter:QPainter,ppath:QPainterPath):
        # 矩形高度
        rect_h = self.height() - self._arrows_h
        rect_w = self.width() - self._arrows_h
        # 画三角
        line_w = self._arrows_h  # 线宽
        lien_x = self.width() // 2 - line_w // 2  # 线的位置-水平
        line_y = self.height()//2  # 线的位置-垂直

        # 绘制文字
        f = QFont()
        f.setPointSize(self.get_fontSize())
        painter.setFont(f)
        # 文字大小
        fs = textSize(f, self._text)
        fw = fs.width()
        fh = fs.height()

        self.textExtend()

        # 画刷
        bru = QBrush(self.get_backgroundColor())
        painter.setBrush(bru)

        # 画笔
        painter.setPen(qt.NoPen)

        if self.__direction == BubbleWidget.Top:
            # 画三角
            ploys = [QPointF(lien_x,self._arrows_h,),QPointF(lien_x+line_w//2,1),
                     QPointF(lien_x+line_w,self._arrows_h)
                     ]
            # 画矩形
            rect = QRect(self.get_margin(),self._arrows_h+self.get_margin(),self.width()-self.get_margin()*2,rect_h-self.get_margin()*2)
            # 文字位置
            x = self.width() // 2 - fw // 2
            y = rect_h//2 + fh//2+self._arrows_h
        elif self.__direction == BubbleWidget.Left:
            # 画三角
            ploys = [QPointF(1,line_y),QPointF(self._arrows_h,line_y-line_w//2),
                     QPointF(self._arrows_h,line_y+line_w//2)
            ]
            rect = QRect(self._arrows_h+self.get_margin(),self.get_margin(),rect_w-self.get_margin()*2,self.height()-self.get_margin()*2)
            # 文字位置
            x = self.width()//2-fw//2 + self._arrows_h//2
            y = self.height()//2 + fh//2
        elif self.__direction == BubbleWidget.Right:
            # 画三角
            ploys =[QPointF(rect_w,line_y-line_w//2),QPointF(rect_w+self._arrows_h,line_y),
                    QPointF(rect_w,line_y+line_w//2)
            ]
            rect = QRect(self.get_margin(),self.get_margin(),rect_w-self.get_margin()*2,self.height()-self.get_margin()*2)
            # 文字位置
            x = (self.width()-self._arrows_h)//2 - fw//2
            y = self.height()//2 + fh//2
        else: # Down
            rect = QRect(self.get_margin(),self.get_margin(),self.width()-self.get_margin()*2,rect_h-self.get_margin()*2)
            # 画三角
            ploys =[QPointF(lien_x,rect_h),
                    QPointF(lien_x+line_w//2,rect_h+self._arrows_h),
                    QPointF(lien_x+line_w,rect_h)]
            # 文字位置
            x = self.width() // 2 - fw//2
            y = rect_h // 2 + fh//2
        # 绘制矩形
        painter.drawRoundedRect(rect, self.get_radius(), self.get_radius())
        # 绘制三角形
        ppath.addPolygon(QPolygonF(ploys))
        painter.fillPath(ppath, self.get_backgroundColor())
        # 绘制文字
        op = QPen()
        op.setColor(QColor(255, 255, 255))
        painter.setPen(op)
        painter.drawText(x,y, self._text)

    def paintEvent(self, e:QPaintEvent) -> None:
        painter = QPainter(self)
        painter.setRenderHints(qt.Antialiasing | qt.SmoothPixmapTransform | qt.TextAntialiasing)

        ppath = QPainterPath()

        # 绘制气泡
        self.drawBubble(painter,ppath)

        painter.end()

    # 专属QSS
    arrowsSize = pyqtProperty(int,fset=set_Arrows,fget=get_Arrows)



# -*- coding:utf-8 -*-
# @time:2023/5/1111:00
# @author:LX
# @file:palettetools.py
# @software:PyCharm

import math

from PyQtGuiLib.core.switchButtons.swButton import SwitchButton
from PyQtGuiLib.header import (
    PYQT_VERSIONS,
    sys,
    QApplication,
    QWidget,
    QVBoxLayout,
    QHBoxLayout,
    QPushButton,
    QLabel,
    Qt,
    QSizePolicy,
    QSpacerItem,
    QLineEdit,
    QFrame,
    QSlider,
    QFormLayout,
    QPixmap,
    qt,
    QPainter,
    QGradient,
    QLinearGradient,
    QRadialGradient,
    QConicalGradient,
    QColor,
    QRect,
    Signal,
    QStackedWidget,
    QCursor,
    QTimer,
    QPoint,
    QMouseEvent,
    QMenu,
    QAction,
    QColorDialog,
    QResizeEvent,
    QWheelEvent,
    QTabWidget,
    QTextBrowser
)
from PyQtGuiLib.styles.superPainter.superPainter import SuperPainter, VirtualObject

Handle_Linear = "linear"
Handle_Radial = "radial"
Handle_Conical = "conical"


class ColorHsv(QFrame):
    rgbaChange = Signal(QColor)

    def __init__(self):
        super().__init__()
        self.resize(300, 50)

        self.suppainter = SuperPainter()

    # 创建HSV颜色条
    def createHuePixmap(self):
        painter = QPainter(self.pix)
        painter.setRenderHints(qt.Antialiasing | qt.SmoothPixmapTransform)  # 抗锯齿
        gradient = QLinearGradient(self.width(), 0, 0, 0)

        i = 0.0
        gradient.setColorAt(0, QColor.fromHsvF(0, 1, 1, 1))
        while i < 1.0:
            gradient.setColorAt(i, QColor.fromHsvF(i, 1, 1, 1))
            i += 1.0 / 16.0
        gradient.setColorAt(1, QColor.fromHsvF(0, 1, 1, 1))
        painter.setPen(qt.NoPen)
        painter.setBrush(gradient)
        painter.drawRoundedRect(self.rect(), 0, 0)

    def mouseMoveEvent(self, e) -> None:
        x = e.pos().x()
        cursor = self.suppainter.virtualObj("cursor")  # 获取虚拟对象
        y,w = cursor.getVirtualArgs()[1:3]
        if 0 <= x and x <= self.width() - w:
            cursor.move(x, y)
            self.update()
            color = self.pix.toImage().pixelColor(x, y)
            self.rgbaChange.emit(color)
        super().mouseMoveEvent(e)

    def resizeEvent(self, e) -> None:
        self.pix = QPixmap(self.size())
        self.pix.fill(qt.transparent)
        self.createHuePixmap()
        if self.suppainter.isVirtualObj("cursor"):
            height = e.size().height()
            cursor = self.suppainter.virtualObj("cursor")
            old_rect = list(cursor.getVirtualArgs())
            old_rect[-1] = height
            cursor.updateArgs(*old_rect)
        super().resizeEvent(e)

    def setColor(self, color: QColor):
        # 计算颜色在颜色条中的位置
        h, s, v, a = color.getHsv()
        x = h / 360 * self.width()
        y = self.suppainter.virtualObj("cursor").getVirtualArgs()[1]
        self.suppainter.virtualObj("cursor").move(x, y)
        self.update()

    def mousePressEvent(self, e) -> None:
        cursor = self.suppainter.virtualObj("cursor")  # 获取虚拟对象
        y = cursor.getVirtualArgs()[1]
        cursor.move(e.pos().x(), y)
        cursor.updateIndexToArgs(2, 4)
        self.update()
        color = self.pix.toImage().pixelColor(e.pos().x(), y)
        self.rgbaChange.emit(color)
        super().mousePressEvent(e)

    def mouseReleaseEvent(self, e) -> None:
        cursor = self.suppainter.virtualObj("cursor")
        cursor.updateIndexToArgs(2, 1)
        self.update()
        super().mouseReleaseEvent(e)

    def paintEvent(self, e) -> None:
        self.suppainter.begin(self)
        # 创建出游标,并设置虚拟对象
        self.suppainter.drawPixmap(e.rect(), self.pix)
        self.suppainter.drawRect(0, 0, 1, self.height(), openAttr={"color": "white", "width": 1},
                                 virtualObjectName="cursor")

        self.suppainter.end()


class MaskWidget(QWidget):
    def __init__(self, timer):
        super().__init__()
        self.setWindowFlags(qt.FramelessWindowHint)
        self.setWindowOpacity(0.1)
        self.__timer = timer
        self.setCursor(Qt.CrossCursor)

    def mousePressEvent(self, e) -> None:
        self.__timer.stop()
        self.close()
        super().mousePressEvent(e)


# Color plate
class ColorBar(QFrame):
    rgbaChange = Signal(QColor)

    def __init__(self):
        super().__init__()
        self.__bgcolor = QColor(0, 255, 0, 255)

        self.is_half = False

        # 当前位置
        self.__cur_pos = (20, 20)
        self.__cur_pos_percentage = (0.1, 0.1)

        self.suppainter = SuperPainter()

    def setAlpha(self, a: int):
        self.__bgcolor.setAlpha(a)
        self.colorLayer()
        self.rgbaChange.emit(self.__bgcolor)

    def setBgColor(self, color: QColor):
        # Synchronous transparency
        old_a = self.__bgcolor.getRgb()[-1]
        color.setAlpha(old_a)
        self.__bgcolor = color
        self.colorLayer()
        self.rgbaChange.emit(color)

    def bgColor(self) -> QColor:
        return self.__bgcolor

    # 灰色图层
    def grayLayer(self):
        if not hasattr(self, "gray_pix"):
            self.gray_pix = QPixmap(self.size())
        else:
            self.gray_pix = self.gray_pix.scaled(self.size())
        self.gray_pix.fill(qt.transparent)
        self.createGrayPixmap()

    # 彩色图层
    def colorLayer(self):
        if not hasattr(self, "pix"):
            self.pix = QPixmap(self.size())
        else:
            self.pix = self.pix.scaled(self.size())
        self.pix.fill(qt.transparent)
        self.createPixmap()

    def createGrayPixmap(self):
        painter = QPainter(self.gray_pix)
        painter.setRenderHints(qt.Antialiasing)

        gradient = QLinearGradient(0, 0, 0, self.height())
        gradient.setColorAt(0, QColor(0, 0, 0, 0))
        gradient.setColorAt(1, QColor("#000"))
        painter.setPen(qt.NoPen)
        painter.setBrush(gradient)
        painter.drawRoundedRect(self.rect(), 2, 2)

    def createPixmap(self):
        painter = QPainter(self.pix)
        painter.setRenderHints(qt.Antialiasing)

        gradient = QLinearGradient(0, 0, self.width(), 0)
        gradient.setColorAt(0, QColor("#fff"))
        gradient.setColorAt(1, self.bgColor())

        painter.setPen(qt.NoPen)
        painter.setBrush(gradient)
        painter.drawRoundedRect(self.rect(), 2, 2)

    def __updateCursorPos(self, pos):
        cursor = self.suppainter.virtualObj("cursor")
        x, y = pos.x() - 10, pos.y() - 10
        self.__cur_pos = (x, y)
        self.__cur_pos_percentage = round(x / self.width(), 2), round(y / self.height(), 2)
        cursor.move(x, y)
        pixmap = self.grab()
        color = pixmap.toImage().pixelColor(pos)
        self.rgbaChange.emit(color)
        if y >= self.height() // 2:
            cursor.updateOpenAttr({"color": "white", "w": 2})
        else:
            cursor.updateOpenAttr({"color": "black", "w": 2})
        self.update()

    def mouseMoveEvent(self, e):
        self.__updateCursorPos(e.pos())
        super().mouseMoveEvent(e)

    def curColor(self) -> QColor:
        # return self.pix.toImage().pixelColor(*self.__cur_pos)
        return self.bgColor()

    def mousePressEvent(self, e):
        cursor = self.suppainter.virtualObj("cursor")
        data = [(2, 20),(3, 20),(4, 10),(5, 10)]
        for i,v in data:
            cursor.updateIndexToArgs(i, v)
        self.__updateCursorPos(e.pos())
        super().mousePressEvent(e)

    def mouseReleaseEvent(self, e:QResizeEvent):
        cursor = self.suppainter.virtualObj("cursor")
        data = [(2, 10), (3, 10), (4, 5), (5, 5)]
        for i,v in data:
            cursor.updateIndexToArgs(i, v)
        self.update()
        super().mouseReleaseEvent(e)

    def paintEvent(self, e) -> None:
        self.suppainter.begin(self)
        self.suppainter.setRenderHints(qt.Antialiasing)
        rect = e.rect()
        self.suppainter.drawPixmap(rect, self.pix)
        self.suppainter.drawPixmap(rect, self.gray_pix)
        self.suppainter.drawRoundedRect(20, 20, 10, 10, 5, 5, openAttr={"color": "black", "w": 2},
                                        virtualObjectName="cursor")
        self.suppainter.end()

    def resizeEvent(self, e) -> None:
        self.grayLayer()
        self.colorLayer()
        if self.suppainter.isVirtualObj("cursor"):
            w,h = e.size().width(),e.size().height()
            cursor = self.suppainter.virtualObj("cursor")
            cursor.move(self.__cur_pos_percentage[0] * w,
                        self.__cur_pos_percentage[1] * h)

        super().resizeEvent(e)


class CodeQSS:
    def __init__(self, code_type="qss"):
        self.code_type = code_type
        self.grab_type = Handle_Linear
        self.grab_mode = "pad"

        self.structure = {
            "head": "qlineargradient({}",
            "pos": "",
            "color": ""
        }

    def setGrab(self, g_type):
        self.grab_type = g_type

    def setGrabMode(self, mode):
        self.grab_mode = mode

    def posArgc(self) -> str:
        return self.structure["pos"]

    def setColorCompound(self, g_color):
        temp = ""
        for i, c in enumerate(g_color):
            color = QColor(g_color[c]["color"])
            temp += "stop: {} rgba({},{},{},{}),".format(i, *color.getRgb())
        self.structure["color"] = temp[:-1] + ");"

    def colorCompound(self) -> str:
        return self.structure["color"]

    def build(self) -> str:
        if self.grab_type != Handle_Conical:
            if self.grab_mode == QGradient.RepeatSpread:
                mode = "repeat"
            elif self.grab_mode == QGradient.ReflectSpread:
                mode = "reflect"
            else:
                mode = "pad"
            head = self.structure["head"].format("spread:" + mode + ",")
        else:
            head = self.structure["head"].format("")
        pos = self.posArgc() + ","
        return head + pos + self.colorCompound()


class CodeQSSLinear(CodeQSS):
    def __init__(self):
        super().__init__()

    def setPosArgc(self, x1, y1, x2, y2):
        t = "x1:{},y1:{},x2:{},y2:{}"
        self.structure["pos"] = t.format(x1, y1, x2, y2)


class CodeQSSRadial(CodeQSS):
    def __init__(self):
        super().__init__()
        self.structure["head"] = "qradialgradient({}"

    def setPosArgc(self, cx, cy, r, fx, fy):
        t = "cx:{},cy:{},radius:{},fx:{},fy:{}"
        self.structure["pos"] = t.format(cx, cy, r, fx, fy)


class CodeQSSConical(CodeQSS):
    def __init__(self):
        super().__init__()
        self.grab_type = Handle_Conical
        self.structure["head"] = "qconicalgradient({}"

    def setPosArgc(self, cx, cy, angle):
        t = "cx:{},cy:{},angle:{}"
        self.structure["pos"] = t.format(cx, cy, angle)


class PLCode:
    def __init__(self, pl="python"):
        self.g_type = Handle_Linear
        self.programming_language = pl
        self.grab_mode = "pad"

        self.structure = {
            "argc": "",
            "color": ""
        }

    def setGrab(self, g_type):
        self.g_type = g_type

    def setGrabMode(self, mode):
        self.grab_mode = mode

    def varName(self) -> str:
        if self.g_type == Handle_Linear:
            return "linear"
        elif self.g_type == Handle_Radial:
            return "radial"
        elif self.g_type == Handle_Conical:
            return "conical"
        else:
            return ""

    def head(self) -> str:
        if self.programming_language == "python":
            if self.g_type == Handle_Linear:
                return "linear = QLinearGradient"
            elif self.g_type == Handle_Radial:
                return "radial = QRadialGradient"
            elif self.g_type == Handle_Conical:
                return "conical = QConicalGradient"
        elif self.programming_language == "cpp":
            if self.g_type == Handle_Linear:
                return "QLinearGradient linear"
            elif self.g_type == Handle_Radial:
                return "QRadialGradient radial"
            elif self.g_type == Handle_Conical:
                return "QConicalGradient conical"
        return ""

    def getMode(self) -> str:
        symbol = ""
        if self.programming_language == "python":
            symbol = "."
        elif self.programming_language == "cpp":
            symbol = "::"

        if self.grab_mode == QGradient.PadSpread:
            return "QGradient{}PadSpread".format(symbol)
        elif self.grab_mode == QGradient.RepeatSpread:
            return "QGradient{}RepeatSpread".format(symbol)
        elif self.grab_mode == QGradient.ReflectSpread:
            return "QGradient{}ReflectSpread".format(symbol)
        return ""

    def setArgc(self, argc: str):
        self.structure["argc"] = argc

    def getArgc(self) -> str:
        return self.structure["argc"]

    def setColorCompound(self, g_color) -> str:
        temp = ""
        for c in g_color:
            c = g_color[c]
            colorScope = c["colorScope"]
            color = "QColor({},{},{},{})".format(*QColor(c["color"]).getRgb())
            temp += "{}.setColorAt({},{})\n".format(self.varName(), colorScope, color)
        self.structure["color"] = temp

    def colorCompound(self) -> str:
        return self.structure["color"]

    def build(self) -> str:
        code = "{}({})\n".format(self.head(), self.getArgc())
        if self.g_type != Handle_Conical:
            code += "{}.setSpread({})\n".format(self.varName(), self.getMode())
        code += self.colorCompound()
        return code


# 展示代码的面板
class VideCode(QTabWidget):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.setWindowFlags(Qt.Tool)
        self.resize(520, 380)
        self.setWindowTitle("View Code")

        self.__qss_widget = QTextBrowser()
        self.__py_widget = QTextBrowser()
        self.__cpp_widget = QTextBrowser()

        self.addTab(self.__qss_widget, "QSS")
        self.addTab(self.__py_widget, "Python Qt")
        self.addTab(self.__cpp_widget, "C++ Qt")

    def setQSSCode(self, qss: str):
        self.__qss_widget.clear()
        self.__qss_widget.setText(qss)
        self.update()

    def setPyCode(self, code: str):
        self.__py_widget.clear()
        self.__py_widget.setText(code)
        self.update()

    def setCppCode(self, code: str):
        self.__cpp_widget.clear()
        self.__cpp_widget.setText(code)


# 颜色句柄
class Handle:
    def __init__(self, x, y, w, h, r=10, lh=8):
        '''

        :param x:
        :param y:
        :param w:
        :param h:
        :param r: 圆角半径
        :param lh: 游标线长
        '''
        self.x = x
        self.y = y
        self.w = w
        self.h = h
        self.r = r
        self.lh = lh

    def rect(self) -> QRect:
        return QRect(self.x, self.y, self.w, self.h)

    def getRect(self) -> tuple:
        return self.x, self.y, self.w, self.h

    def radius(self) -> int:
        return self.r

    def lineH(self) -> int:
        return self.lh


# 渐变信息类
class GradientInfo:
    class InfoABC:
        def __init__(self):
            self.info = {
                "pos": [0, 50, 0, 50],
                "gColor": {
                    # handle_n is the specified naming rule
                    "handle_1": {
                        "colorScope": 0,
                        "color": qt.red
                    },
                    "handle_2": {
                        "colorScope": 1,
                        "color": qt.blue
                    }
                },
                # The linear gradient has only two handles
                "handle": [
                    {
                        "vobj": "linear_1",
                        "handle": Handle(5, 5, 16, 16, 8, 40),
                        "openAttr": {"c": "#000", "w": 2},
                        "brushAttr": {"c": qt.red},
                        "pos_percentage": (0.0, 0.0)
                    },
                    {
                        "vobj": "linear_2",
                        "handle": Handle(335, 5, 16, 16, 8, 40),
                        "openAttr": {"c": "#000", "w": 2},
                        "brushAttr": {"c": qt.blue},
                        "pos_percentage": (0.0, 0.0)
                    }
                ]
            }
            self.max_hand_id = 2

        def idAdd(self) -> int:
            self.max_hand_id += 1
            return self.max_hand_id

        def updateHandPos(self, hand_vobj: str, pos_percentage):
            pass

        def getPos(self) -> list:
            return self.info["pos"]

        def colorCount(self) -> int:
            return len(self.info["gColor"])

        def Colors(self) -> dict:
            return self.info["gColor"]

        def updateStart(self, e):
            self.info["pos"][0] = e.x()
            self.info["pos"][1] = e.y()

        def updateSpread(self, e):
            self.info["pos"][2] = e.x()
            self.info["pos"][3] = e.y()

        def updateColor(self, hand_id: str, colorScope=None, color=None):
            if colorScope:
                self.Colors()[hand_id]["colorScope"] = colorScope
            if color:
                self.Colors()[hand_id]["color"] = color

        def handle(self) -> list:
            return self.info["handle"]

        def appendColor(self, colorScope, color):
            name = "handle_{}".format(self.idAdd())
            self.info["gColor"][name] = {
                "colorScope": colorScope,
                "color": color
            }

        def removeHande(self, hand_id):
            del self.Colors()[hand_id]

    class Linear(InfoABC):
        def __init__(self):
            super().__init__()
            self.info = {
                "pos": [0, 50, 330, 50],
                "gColor": {
                    # handle_n is the specified naming rule
                    "handle_1": {
                        "colorScope": 0,
                        "color": qt.red
                    },
                    "handle_2": {
                        "colorScope": 1,
                        "color": qt.blue
                    }
                },
                # The linear gradient has only two handles
                "handle": [
                    {
                        "vobj": "linear_1",
                        "handle": Handle(0, 50, 16, 16, 8, 40),
                        "openAttr": {"c": "#000", "w": 2},
                        "brushAttr": {"c": qt.red},
                        "pos_percentage": (0.0, 0.0)
                    },
                    {
                        "vobj": "linear_2",
                        "handle": Handle(335, 50, 16, 16, 8, 40),
                        "openAttr": {"c": "#000", "w": 2},
                        "brushAttr": {"c": qt.blue},
                        "pos_percentage": (0.9, 0.0)
                    }
                ]
            }

        def updateHandPos(self, hand_vobj: str, pos_percentage):
            if hand_vobj == "linear_1":
                self.handle()[0]["pos_percentage"] = pos_percentage
            if hand_vobj == "linear_2":
                self.handle()[1]["pos_percentage"] = pos_percentage

        def updateSize(self, w, h):
            self.info["pos"][2] = w
            self.info["pos"][3] = h

    class Radial(InfoABC):
        def __init__(self):
            super().__init__()
            self.info = {
                "pos": [100, 100, 50, 80, 80],
                "gColor": {
                    # handle_n is the specified naming rule
                    "handle_1": {
                        "colorScope": 0,
                        "color": qt.white
                    },
                    "handle_2": {
                        "colorScope": 1,
                        "color": qt.black
                    }
                },
                "handle": [
                    {
                        "vobj": "radial_1",
                        "handle": Handle(100, 100, 16, 16, 8, 40),
                        "openAttr": {"c": "#000", "w": 2},
                        "brushAttr": {"c": qt.red},
                        "pos_percentage": (0.0, 0.0)
                    },
                    {
                        "vobj": "radial_2",
                        "handle": Handle(80, 80, 14, 14, 7, 40),
                        "openAttr": {"c": "#000", "w": 2},
                        "brushAttr": {"c": qt.blue},
                        "pos_percentage": (0.0, 0.0)
                    },
                    {
                        "vobj": "radial_3",
                        "handle": Handle(120, 80, 14, 14, 7, 40),
                        "openAttr": {"c": "#000", "w": 2},
                        "brushAttr": {"c": qt.gray},
                        "pos_percentage": (0.0, 0.0)
                    }

                ]
            }

        def updateHandPos(self, hand_vobj: str, pos_percentage):
            if hand_vobj == "radial_1":
                self.handle()[0]["pos_percentage"] = pos_percentage
            if hand_vobj == "radial_2":
                self.handle()[1]["pos_percentage"] = pos_percentage

        def updateCenterPos(self, e):
            if isinstance(e,QPoint):
                self.info["pos"][0] = e.x()
                self.info["pos"][1] = e.y()
            else:
                self.info["pos"][0] = e.pos().x()
                self.info["pos"][1] = e.pos().y()

        def updateCenterPos2(self, e):
            if isinstance(e,QPoint):
                self.info["pos"][3] = e.x()
                self.info["pos"][4] = e.y()
            else:
                self.info["pos"][3] = e.pos().x()
                self.info["pos"][4] = e.pos().y()

        # 更新外圈大小
        def updateOuterSize(self, n: int):
            self.info["pos"][2] = n

    class Conical(InfoABC):
        def __init__(self):
            super().__init__()
            self.info = {
                "pos": [108, 108, 90],
                "gColor": {
                    # handle_n is the specified naming rule
                    "handle_1": {
                        "colorScope": 0,
                        "color": qt.red
                    },
                    "handle_2": {
                        "colorScope": 1,
                        "color": qt.blue
                    }
                },
                "handle": [
                    {
                        "vobj": "conical_1",
                        "handle": Handle(100, 100, 16, 16, 8, 40),
                        "openAttr": {"c": "#000", "w": 2},
                        "brushAttr": {"c": qt.red},
                        "pos_percentage": (0.0, 0.0)
                    },
                    {
                        "vobj": "conical_2",
                        "handle": Handle(80, 80, 14, 14, 7, 40),
                        "openAttr": {"c": "#000", "w": 2},
                        "brushAttr": {"c": qt.blue},
                        "pos_percentage": (0.0, 0.0)
                    }

                ]
            }

        def updateHandPos(self, hand_vobj: str, pos_percentage):
            if hand_vobj == "conical_1":
                self.handle()[0]["pos_percentage"] = pos_percentage
            if hand_vobj == "conical_2":
                self.handle()[1]["pos_percentage"] = pos_percentage

        def updateCenter(self, e):
            if isinstance(e,QPoint):
                self.info["pos"][0] = e.x()
                self.info["pos"][1] = e.y()
            else:
                self.info["pos"][0] = e.pos().x()
                self.info["pos"][1] = e.pos().y()

        def updateAngle(self, a: int):
            self.info["pos"][2] = a

    def __init__(self):
        self.__linear = self.Linear()
        self.__radial = self.Radial()
        self.__conical = self.Conical()

    def getLinear(self) -> Linear:
        return self.__linear

    def getRadial(self) -> Radial:
        return self.__radial

    def getConical(self) -> Conical:
        return self.__conical

    def getObj(self, g_type):
        if g_type == Handle_Linear:
            return self.getLinear()
        elif g_type == Handle_Radial:
            return self.getRadial()
        elif g_type == Handle_Conical:
            return self.getConical()
        return None

    def updatePos(self, g_type: str, hand_id: str, color_scope=None):
        self.getObj(g_type).updateColor(hand_id, colorScope=color_scope)

    def updateColor(self, g_type: str, hand_id: str, color=None):
        self.getObj(g_type).updateColor(hand_id, color=color)

    def newColor(self, g_type: str, colorScope, color):
        self.getObj(g_type).appendColor(colorScope, color)

    def delColor(self, g_type: str, hand_id: str):
        self.getObj(g_type).removeHande(hand_id)


# 渐变变板
class GradientWidget(QFrame):
    gradientQSSCoded = Signal(str)

    def __init__(self, grab_type="conical", *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.suppainter = SuperPainter()

        self.grab_type = grab_type
        self.grab_mode = QGradient.PadSpread

        self.gradientInfo = GradientInfo()

        self.isHideHand = False

        # -- QSS or Programming language Generate
        self.code_linear = CodeQSSLinear()
        self.code_radial = CodeQSSRadial()
        self.code_conical = CodeQSSConical()
        self.pl_code_py = PLCode("python")
        self.pl_code_cpp = PLCode("cpp")
        # ---

        self.updateAllPix()
        self.createHandle()

    def setViewCode(self, obj):
        self.view_code = obj

    def setGrabType(self, g_type: str):
        self.grab_type = g_type

    def grabType(self) -> str:
        return self.grab_type

    def setGrabMode(self, mode: str):
        if mode == "pad":
            self.grab_mode = QGradient.PadSpread
        if mode == "repeat":
            self.grab_mode = QGradient.RepeatSpread
        if mode == "reflect":
            self.grab_mode = QGradient.ReflectSpread

        self.updateAllPix()

    def createPix(self, g_type):
        if g_type == Handle_Linear:
            if not hasattr(self, "linear_pix"):
                self.linear_pix = QPixmap(self.size())
            else:
                self.linear_pix = self.linear_pix.scaled(self.size())
            obj_pix = self.linear_pix
            info_obj = self.gradientInfo.getLinear()
            obj_g = QLinearGradient(*info_obj.getPos())
        elif g_type == Handle_Radial:
            if not hasattr(self, "radia_pix"):
                self.radia_pix = QPixmap(self.size())
                self.radia_outer_pix = QPixmap(self.size())
            else:
                self.radia_pix = self.radia_pix.scaled(self.size())
                self.radia_outer_pix = self.radia_outer_pix.scaled(self.size())
            self.radia_outer_pix.fill(qt.transparent)
            obj_pix = self.radia_pix
            info_obj = self.gradientInfo.getRadial()
            obj_g = QRadialGradient(*info_obj.getPos())
        elif g_type == Handle_Conical:
            if not hasattr(self, "conical_pix"):
                self.conical_pix = QPixmap(self.size())
            else:
                self.conical_pix = self.conical_pix.scaled(self.size())
            obj_pix = self.conical_pix
            info_obj = self.gradientInfo.getConical()
            obj_g = QConicalGradient(*info_obj.getPos())
        else:
            obj_pix = None
            obj_g = None
            info_obj = None

        if obj_pix and obj_g and info_obj:
            obj_pix.fill(qt.transparent)

            obj_g.setSpread(self.grab_mode)
            for c in info_obj.Colors().values():
                obj_g.setColorAt(c["colorScope"], c["color"])

            painter = QPainter(obj_pix)
            painter.setRenderHints(qt.Antialiasing | qt.SmoothPixmapTransform)

            painter.setPen(qt.NoPen)
            painter.setBrush(obj_g)
            painter.drawRect(self.rect())

    def getPixObj(self, g_type) -> QPixmap:
        if g_type == Handle_Linear:
            return self.linear_pix
        elif g_type == Handle_Radial:
            return self.radia_pix
        elif g_type == Handle_Conical:
            return self.conical_pix
        else:
            return None

    def updateAllPix(self):
        for g_type in [Handle_Linear, Handle_Radial, Handle_Conical]:
            self.createPix(g_type)
        self.update()

    def createHandle(self):
        if self.grab_type in [Handle_Linear, Handle_Radial, Handle_Conical]:
            self.cur_checked_hand = None

    # ---- 同步  渐变 外部事件 - 触发接口 ---
    def syncPos(self, g_type, hand_id, color_scope):
        self.gradientInfo.updatePos(g_type, hand_id, color_scope=color_scope)
        self.createPix(g_type)
        self.update()

    def syncColor(self, g_type, hand_id, color):
        self.gradientInfo.updateColor(g_type, hand_id, color=color)
        self.createPix(g_type)
        self.update()

    def syncNewColor(self, g_type, colorScope, color):
        self.gradientInfo.newColor(g_type, colorScope, color)
        self.createPix(g_type)
        self.update()

    def syncDelColor(self, g_type, hand_id: str):
        self.gradientInfo.delColor(g_type, hand_id)
        self.createPix(g_type)
        self.update()

    # --------同步  渐变 外部事件 - 触发接口 ---

    # --------生成代码
    def buildCodeQSS(self, g_type):
        if g_type == Handle_Linear:
            obj = self.code_linear
            x1, y1, x2, y2 = self.gradientInfo.getLinear().getPos()
            x1, y1 = round(x1 / self.width(), 3), round(y1 / self.height(), 3)
            x2, y2 = round(x2 / self.width(), 3), round(y2 / self.height(), 3)
            argc = x1, y1, x2, y2
        elif g_type == Handle_Radial:
            obj = self.code_radial
            cx, cy, r, fx, fy = self.gradientInfo.getRadial().getPos()
            cx, cy = round(cx / self.width(), 3), round(cy / self.height(), 3)
            fx, fy = round(fx / self.width(), 3), round(fy / self.height(), 3)
            w = self.suppainter.virtualObj("radial_3").getVirtualArgs()[0]
            rr = round(w / self.width(), 2)
            argc = cx, cy, rr, fx, fy
        elif g_type == Handle_Conical:
            obj = self.code_conical
            cx, cy, angle = self.gradientInfo.getConical().getPos()
            cx, cy = round(cx / self.width(), 3), round(cy / self.height(), 3)
            argc = cx, cy, int(math.fabs(angle))
        else:
            obj = None

        if obj:
            obj.setGrabMode(self.grab_mode)
            obj.setPosArgc(*argc)
            obj.setColorCompound(self.gradientInfo.getObj(g_type).Colors())
            g_qss = obj.build()
            self.gradientQSSCoded.emit(g_qss) # send single
            self.view_code.setQSSCode(g_qss)

    def buildCodePL(self, pl, obj, g_type):
        if pl == "python":
            code_obj = self.pl_code_py
        elif pl == "cpp":
            code_obj = self.pl_code_cpp
        else:
            code_obj = None

        if code_obj:
            code_obj.setGrab(g_type)
            code_obj.setGrabMode(self.grab_mode)
            if g_type == Handle_Conical:
                argc = "{},{},{}".format(*obj.getPos())
            elif g_type == Handle_Radial:
                argc = "{},{},{},{},{}".format(*obj.getPos())
            else:
                argc = "{},{},{},{}".format(*obj.getPos())
            code_obj.setArgc(argc)
            code_obj.setColorCompound(obj.Colors())
            if pl == "python":
                self.view_code.setPyCode(code_obj.build())
            elif pl == "cpp":
                self.view_code.setCppCode(self.pl_code_cpp.build())

    def mouseMoveEvent(self, e: QMouseEvent) -> None:
        if self.cur_checked_hand and self.isHideHand is False:
            hand = self.cur_checked_hand  # type:VirtualObject
            x, y = e.x()-8, e.y()-8
            hand.move(x, y)

            p_x, p_y = round(x / self.width(), 2), round(y / self.height(), 2)

            if self.rect().contains(e.pos()):
                if self.grab_type == Handle_Linear:
                    info_obj = self.gradientInfo.getLinear()
                    if hand.virName() == "linear_1":
                        info_obj.updateStart(e)
                    elif hand.virName() == "linear_2":
                        info_obj.updateSpread(e)
                elif self.grab_type == Handle_Radial:
                    info_obj = self.gradientInfo.getRadial()
                    if hand.virName() == "radial_1":
                        info_obj.updateCenterPos(e)
                    if hand.virName() == "radial_2":
                        info_obj.updateCenterPos2(e)
                    if hand.virName() == "radial_3":
                        # The distance between any two points on the rectangle
                        x1, y1 = info_obj.getPos()[:2]
                        sqrt_n = int(math.sqrt(math.pow(x - x1, 2) + math.pow(y - y1, 2)))
                        info_obj.updateOuterSize(sqrt_n)
                elif self.grab_type == Handle_Conical:
                    info_obj = self.gradientInfo.getConical()
                    if hand.virName() == "conical_1":
                        info_obj.updateCenter(e)
                    if hand.virName() == "conical_2":
                        # Any point of the rectangle, the formula for radians
                        x1, y1 = info_obj.getPos()[:2]
                        angle_arc = math.atan2(y - y1, x - x1)
                        angle = angle_arc * 180 / math.pi
                        info_obj.updateAngle(int(-angle))
                info_obj.updateHandPos(hand.virName(), (p_x, p_y))
                # qss code and pl code
                self.buildCodeQSS(self.grab_type)
                self.buildCodePL("python", info_obj, self.grab_type)
                self.buildCodePL("cpp", info_obj, self.grab_type)
                # qss ---- up
                self.createPix(self.grab_type)
                self.update()
        super().mouseMoveEvent(e)

    def mousePressEvent(self, e: QMouseEvent) -> None:
        if e.buttons() == Qt.LeftButton:
            hand_obj = self.gradientInfo.getObj(self.grab_type).handle()
            if hand_obj and self.isHideHand is False:
                for hand in hand_obj:
                    c_hand = self.suppainter.virtualObj(hand["vobj"])
                    if c_hand.isClick(e):
                        c_hand.updateOpenAttr({"c": "#fff", "w": 3})
                        self.cur_checked_hand = c_hand
                        break
                self.update()
        super().mousePressEvent(e)

    def mouseReleaseEvent(self, e: QMouseEvent) -> None:
        hand_obj = self.gradientInfo.getObj(self.grab_type).handle()
        if hand_obj and self.isHideHand is False:
            for hand in hand_obj:
                c_hand = self.suppainter.virtualObj(hand["vobj"])
                c_hand.updateOpenAttr({"c": "#000", "w": 2})
            self.update()
            self.cur_checked_hand = None
        super().mouseReleaseEvent(e)

    def setHideHand(self, b: bool):
        self.isHideHand = b
        self.update()

    def paintEvent(self, e):
        self.suppainter.begin(self)
        self.suppainter.setRenderHints(qt.Antialiasing | qt.SmoothPixmapTransform)

        pix = self.getPixObj(self.grab_type)
        if pix:
            self.suppainter.drawPixmap(e.rect(), pix)
        handle = self.gradientInfo.getObj(self.grab_type).handle()
        if handle and self.isHideHand is False:
            for hand in handle:
                r = hand["handle"].radius()
                self.suppainter.drawRoundedRect(*hand["handle"].getRect(),
                                                r, r,
                                                openAttr=hand["openAttr"],
                                                brushAttr=hand["brushAttr"],
                                                virtualObjectName=hand["vobj"]
                                                )
        self.suppainter.end()


    def resizeEvent(self, e: QResizeEvent):
        w, h = e.size().width(), e.size().height()
        for hand in self.gradientInfo.getObj(self.grab_type).handle():
            vobj = hand["vobj"]
            if vobj == "radial_3":
                continue
            if self.suppainter.isVirtualObj(vobj):
                p_x, p_y = hand["pos_percentage"]
                cursor = self.suppainter.virtualObj(vobj)
                cursor.move(round(w * p_x, 2), round(h * p_y, 2))

        p_x, p_y = self.gradientInfo.getObj(self.grab_type).handle()[0]["pos_percentage"]
        if self.grab_type == Handle_Linear:
            p_x2, p_y2=self.gradientInfo.getObj(self.grab_type).handle()[1]["pos_percentage"]
            self.gradientInfo.getObj(self.grab_type).updateStart(QPoint(int(w * p_x)+8, int(h * p_y)+8))
            self.gradientInfo.getObj(self.grab_type).updateSpread(QPoint(int(w * p_x2)+8, int(h * p_y2)+8))
        elif self.grab_type == Handle_Radial:
            p_x2, p_y2 = self.gradientInfo.getObj(self.grab_type).handle()[1]["pos_percentage"]
            self.gradientInfo.getObj(self.grab_type).updateCenterPos(QPoint(int(w * p_x) + 8, int(h * p_y) + 8))
            self.gradientInfo.getObj(self.grab_type).updateCenterPos2(QPoint(int(w * p_x2) + 8, int(h * p_y2) + 8))
        elif self.grab_type == Handle_Conical:
            self.gradientInfo.getObj(self.grab_type).updateCenter(QPoint(int(w * p_x)+8, int(h * p_y)+8))
        self.createPix(self.grab_type)
        self.update()
        super().resizeEvent(e)


# -----------------------------


# 渐变通用操作台
class ColorOperation(QFrame):
    syncPosed = Signal(str, str, float)
    syncColor = Signal(str, str, QColor)
    syncNewColor = Signal(str, float, QColor)
    syncDelColor = Signal(str, str)

    def __init__(self, g_type, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.setFixedHeight(80)

        self.suppainter = SuperPainter()

        if g_type == Handle_Radial:
            handle1_color = qt.white
            handle2_color = qt.black
        else:
            handle1_color = qt.red
            handle2_color = qt.blue

        self.handles = [
            {
                "vobj": "handle_1",
                "handle": Handle(5, 5, 20, 20, 10, 40),
                "openAttr": {"c": "#000", "w": 2},
                "brushAttr": {"c": handle1_color},
                "colorScope": 0,
                "color": handle1_color
            },
            {
                "vobj": "handle_2",
                "handle": Handle(335, 5, 20, 20, 10, 40),
                "openAttr": {"c": "#000", "w": 2},
                "brushAttr": {"c": handle2_color},
                "colorScope": 1,
                "color": handle2_color
            }
        ]

        # The id only increases and does not decrease, maintaining uniqueness
        self.max_handle_id = 2

        self.g_type = g_type

        self.cursor_flag = ""
        self.updatePix()

        self.setContextMenuPolicy(Qt.CustomContextMenu)
        self.customContextMenuRequested.connect(self.menu_event)

    def menu_event(self):
        menu_ = QMenu(self)

        new_cursor = QAction("新建游标", self)
        del_cursor = QAction("删除游标", self)
        update_color = QAction("更新颜色", self)
        new_cursor.triggered.connect(self.createCursor_event)
        del_cursor.triggered.connect(self.delCursor_event)
        update_color.triggered.connect(self.updateCursorColor_event)
        menu_.addAction(new_cursor)
        menu_.addAction(del_cursor)
        menu_.addAction(update_color)

        menu_.popup(QCursor.pos())

    def createCursor_event(self):
        if not self._right_pressed_pos:
            return

        x = self._right_pressed_pos.x()
        self.max_handle_id += 1
        structure = {
            "vobj": "handle_{}".format(self.max_handle_id),
            "handle": Handle(x, 5, 20, 20, 10, 40),
            "openAttr": {"c": "#000", "w": 2},
        }
        color = QColorDialog.getColor()
        if color.isValid():
            structure["brushAttr"] = {"c": color}
            structure["colorScope"] = 1 / self.width() * x
            structure["color"] = color
            self.handles.append(structure)
            self.syncNewColor.emit(self.g_type, round(1 / self.width() * x, 2), color)
            self.updatePix()
            self.update()
        self._right_pressed_pos = None

    def getHandle(self, vname: str):
        for info in self.handles:
            if info["vobj"] == vname:
                return info
        return None

    def delCursor_event(self):
        if not self._right_pressed_pos:
            return

        for vname in self.vObjs():
            cursor = self.suppainter.virtualObj(vname)
            if cursor.isClick(self._right_pressed_pos):
                self.handles.remove(self.getHandle(vname))
                #
                self.syncDelColor.emit(self.g_type, vname)
                break

        self.updatePix()
        self.update()
        self._right_pressed_pos = None

    def updateCursorColor_event(self):
        if not self._right_pressed_pos:
            return
        for vname in self.vObjs():
            cursor = self.suppainter.virtualObj(vname)
            if cursor.isClick(self._right_pressed_pos):
                color = QColorDialog.getColor()
                if color.isValid():
                    hand = self.getHandle(vname)
                    hand["brushAttr"]["c"] = color
                    hand["color"] = color
                    self.syncColor.emit(self.g_type, vname, color)
                    break

        self.updatePix()
        self.update()
        self._right_pressed_pos = None

    def vObjs(self) -> list:
        return [vname["vobj"] for vname in self.handles]

    def updatePix(self):
        if not hasattr(self, "pix"):
            self.pix = QPixmap(self.size())
        else:
            self.pix = self.pix.scaled(self.size())
        self.pix.fill(qt.transparent)

        h2 = self.height() // 2
        linear = QLinearGradient(0, h2, self.width(), h2)

        for cursor in self.handles:
            linear.setColorAt(cursor["colorScope"], cursor["color"])

        painter = QPainter(self.pix)
        painter.setRenderHints(qt.Antialiasing | qt.SmoothPixmapTransform)
        painter.setPen(qt.NoPen)
        painter.setBrush(linear)
        painter.drawRoundedRect(self.rect(), 5, 5)

    # 根据游标名称,更新colorScope
    def updateColorScope(self, cursor_name, v):
        for hand in self.handles:
            if hand["vobj"] == cursor_name:
                hand["colorScope"] = v
                break

    def mouseMoveEvent(self, e: QMouseEvent) -> None:
        if self.cursor_flag and e.buttons() == Qt.LeftButton:
            cursor = self.suppainter.virtualObj(self.cursor_flag)
            if e.x() + cursor.getWidth() // 2 >= cursor.getWidth() // 2 \
                    and e.x() <= self.width():
                cursor.updateIndexToArgs(0, e.x() - cursor.getWidth() // 2)
                self.updateColorScope(self.cursor_flag, 1 / self.width() * e.x())
                self.updatePix()
                # 发送 游标信息
                self.syncPosed.emit(self.g_type, self.cursor_flag,
                                    round(1 / self.width() * e.x(), 2))
            self.update()
        super().mouseMoveEvent(e)

    def mousePressEvent(self, e: QMouseEvent) -> None:
        if e.buttons() == Qt.LeftButton:
            for vname in self.vObjs():
                cursor = self.suppainter.virtualObj(vname)
                if cursor.isClick(e):
                    r, b, g, a = QColor(cursor.getVirtualBrushAttr()["c"]).getRgb()
                    reverse_color = QColor(255 - r, 255 - b, 255 - g)
                    cursor.updateOpenAttr({"c": reverse_color, "w": 3})
                    cursor.updateIndexToArgs(5, 50)
                    self.cursor_flag = vname
                    break
                else:
                    self.cursor_flag = ""
            self.update()
        elif e.buttons() == Qt.RightButton:
            self._right_pressed_pos = QPoint(e.pos().x(), e.pos().y())
        super().mousePressEvent(e)

    def mouseReleaseEvent(self, e: QMouseEvent) -> None:
        self.cursor_flag = ""
        for vname in self.vObjs():
            cursor = self.suppainter.virtualObj(vname)
            cursor.updateOpenAttr({"c": "#000", "w": 2})
            cursor.updateIndexToArgs(5, 40)
        self.update()
        super().mouseReleaseEvent(e)

    def paintEvent(self, e):
        self.suppainter.begin(self)
        self.suppainter.setRenderHints(qt.Antialiasing | qt.SmoothPixmapTransform)

        self.suppainter.setPen(qt.NoPen)
        rect = QRect(0, self.height() - 50, self.width(), self.height())
        self.suppainter.drawPixmap(rect, self.pix)

        # 绘制句柄
        for cursor in self.handles:
            handle_obj = cursor["handle"]
            self.suppainter.drawCursor(*handle_obj.getRect(),
                                       handle_obj.radius(),
                                       handle_obj.lineH(),
                                       openAttr=cursor["openAttr"],
                                       brushAttr=cursor["brushAttr"],
                                       virtualObjectName=cursor["vobj"]
                                       )
        self.suppainter.end()

    def resizeEvent(self, e: QResizeEvent) -> None:
        width = e.size().width()
        for hand in self.handles:
            if self.suppainter.isVirtualObj(hand["vobj"]):
                cursor = self.suppainter.virtualObj(hand["vobj"])
                new_pos = width * hand["colorScope"]
                if new_pos >= width:
                    new_pos -= 20
                cursor.updateIndexToArgs(0, new_pos)
        self.updatePix()
        super().resizeEvent(e)


# -------------------------------


# 调色工具
class PaletteTools(QWidget):
    clickColor = Signal(QColor)
    qssColored = Signal(str)

    def __init__(self):
        super().__init__()
        self.setObjectName("widget")
        self.setWindowTitle("Palette")
        self.resize(600, 400)

        # Pipette timer
        self.timer = QTimer()
        self.timer.timeout.connect(self.updateColor)

        self.view_code = VideCode()

        self.setUI()
        self.myEvent()

    def setUI(self):
        self.__lay = QVBoxLayout(self)

        self.__top_lay = QHBoxLayout()
        self.__top_lay.setSpacing(20)
        self.pure_color_btn = QPushButton()  # 纯色按钮
        self.line_btn = QPushButton()  # 线性渐变按钮
        self.radial_btn = QPushButton()  # 径向渐变按钮
        self.conical_btn = QPushButton()  # 角度渐变按钮
        self.pure_color_btn.setObjectName("pure_color_btn")
        self.line_btn.setObjectName("line_btn")
        self.radial_btn.setObjectName("repeat_btn")
        self.conical_btn.setObjectName("reflect_btn")
        self.pure_color_btn.setToolTip("纯色")
        self.line_btn.setToolTip("线性渐变")
        self.radial_btn.setToolTip("径向渐变")
        self.conical_btn.setToolTip("角度渐变")

        #
        self.pad_btn = QPushButton()
        self.rep_btn = QPushButton()
        self.ref_btn = QPushButton()
        self.pad_btn.setObjectName("Pad")
        self.rep_btn.setObjectName("Rep")
        self.ref_btn.setObjectName("Ref")

        self.__spacer = QSpacerItem(0, 0, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.demonstration_color_l = QLabel()

        self.demonstration_color_l.setFixedSize(24, 24)
        self.color_straw_btn = QPushButton()  # 吸管
        self.color_straw_btn.setObjectName("color_straw_btn")
        self.color_straw_btn.setText("吸")
        self.color_straw_btn.setFixedSize(24, 24)
        self.hand_btn = SwitchButton()
        self.hand_btn.setShape(SwitchButton.Shape_Square)
        self.hand_btn.setFixedSize(50, 24)
        self.hex_line = QLineEdit()
        self.hex_line.setFixedSize(100, 24)
        self.hex_line.setText("#00ff00")

        for btn in [self.pure_color_btn, self.line_btn, self.radial_btn, self.conical_btn]:
            btn.setFixedSize(24, 24)
            self.__top_lay.addWidget(btn, )

        for btn in [self.pad_btn, self.rep_btn, self.ref_btn]:
            btn.setFixedSize(48, 24)
            self.__top_lay.addWidget(btn, )

        self.__top_lay.addItem(self.__spacer)
        self.__top_lay.addWidget(self.demonstration_color_l)
        self.__top_lay.addWidget(self.color_straw_btn)
        self.__top_lay.addWidget(self.hand_btn)
        self.__top_lay.addWidget(self.hex_line)

        # 中间层
        self.__middle_lay = QHBoxLayout()
        self.__middle_lay.setContentsMargins(0, 0, 0, 0)
        self.__middle_lay.setSpacing(3)

        # ---

        # 纯色,线性渐变,径向渐变,角度渐变
        self.st = QStackedWidget()
        self.st_pure_color_wideget = QWidget()
        self.st_linear_widget = QWidget()
        self.st_radial_widget = QWidget()
        self.st_conicalt_widget = QWidget()
        self.st.addWidget(self.st_pure_color_wideget)
        self.st.addWidget(self.st_linear_widget)
        self.st.addWidget(self.st_radial_widget)
        self.st.addWidget(self.st_conicalt_widget)

        self.pureColorWidget()
        for g_type in [Handle_Linear,Handle_Radial,Handle_Conical]:
            self.grabColorWidget(g_type)
        # -----

        # 不同颜色的操作太区域
        self.operation_st = QStackedWidget()
        self.operation_st.setFixedWidth(100)

        # 操作台
        self.pureColorOperation()
        self.linearColorOperation()
        self.radialColorOperation()
        self.conicalColorOperation()

        self.__lay.addLayout(self.__top_lay)
        self.__lay.addLayout(self.__middle_lay)

        # 初始化演示色块
        self.setDemonstrationColor(self.color_bar.bgColor())
        self.setStyleSheet('''
#widget{
background-color:#0b4a2d;
}
#color_straw_btn{
font: 10pt "等线";
background-color: rgb(20, 134, 79);
color: rgb(232, 232, 232);
border:1px solid rgb(85, 170, 127);
border-radius:2px;
}
#color_straw_btn:pressed{
background-color:rgb(14, 95, 56)
}
#pure_color_btn,#line_btn,#repeat_btn,#reflect_btn{
border-radius:12px;
border:2px solid rgba(0,0,0,255);
}
#pure_color_btn:hover,#line_btn:hover,#repeat_btn:hover,#reflect_btn:hover{
border:3px solid rgba(0,0,0,255);
}
#pure_color_btn:pressed,#line_btn:pressed,#repeat_btn:pressed,#reflect_btn:pressed{
border:2px solid rgba(0,0,0,255);
}
#pure_color_btn{
background-color: rgb(255, 170, 0);
}
#line_btn{
background-color:qlineargradient(spread:pad,x1:0.002,y1:0.457,x2:0.957,y2:0.463,stop:0.49148936170212765 rgba(217, 155, 9, 255),stop:0.5893617021276596 rgba(173, 166, 147, 255));
}
#repeat_btn{
background-color:qradialgradient(spread:pad,cx:0.436,cy:0.483,radius:0.462,fx:0.47,fy:0.533,stop:0.49148936170212765 rgba(217, 155, 9, 255),stop:0.5893617021276596 rgba(173, 166, 147, 255));
}
#reflect_btn{
background-color:qconicalgradient(cx:0.448, cy:0.42, angle:120 stop:0.49148936170212765 rgba(217, 155, 9, 255),stop:0.5893617021276596 rgba(173, 166, 147, 255));
}
QLabel{
color: rgb(108, 210, 171);
font: 11pt "等线";
}
#colorButton,#linear_view{
border-radius:5px;
background-color: rgb(88, 142, 128);
color: rgb(255, 255, 255);
font: 11pt "等线";
}
QLineEdit{
border:none;
background-color: rgba(23, 154, 93,100);
color: rgb(209, 209, 209);
}

#Pad:hover,#Rep:hover,#Ref:hover{
border:2px solid white;
}
#Pad,#Rep,#Ref,#Pad:pressed,#Rep:pressed,#Ref:pressed{
border:2px solid back;
border-radius:3px;
}
#Pad{
background-color: qlineargradient(spread:pad, x1:0.001, y1:0.471, x2:1, y2:0.482955, stop:0 rgba(232, 200, 97, 255), stop:1 rgba(174, 174, 174, 255));
}
#Rep{
background-color:qlineargradient(spread:repeat, x1:0.33, y1:0.482364, x2:0.63, y2:0.477273, stop:0 rgba(232, 200, 97, 255), stop:1 rgba(174, 174, 174, 255));
}
#Ref{
background-color:qlineargradient(spread:reflect, x1:0.554, y1:0.475, x2:0.63, y2:0.477273, stop:0 rgba(232, 200, 97, 255), stop:1 rgba(174, 174, 174, 255));
}

        ''')

    def updateColor(self):
        if hasattr(self, "maskWidget"):
            self.maskWidget.move(QCursor.pos() - QPoint(50, 50))
        # straw
        pos = QCursor.pos()
        screen = QApplication.primaryScreen()
        if screen is not None:
            pixmap = screen.grabWindow(0, pos.x(), pos.y(), 1, 1)
            color = pixmap.toImage().pixelColor(0, 0)
            self.setDemonstrationColor(color)
            self.setLabelRGB(color)
            self.hex_line.setText(color.name())

    # 纯色面板
    def pureColorWidget(self):
        self.__m_v_lay = QVBoxLayout(self.st_pure_color_wideget)
        self.color_bar = ColorBar()
        self.hsv_widget = QWidget()
        self.hsv_widget.setFixedHeight(50)
        self.__hsv_lay = QVBoxLayout(self.hsv_widget)
        self.hsv = ColorHsv()
        self.hsv.setFixedHeight(15)
        self.splider = QSlider()
        self.splider.setMinimum(0)
        self.splider.setMaximum(255)
        self.splider.setValue(255)
        self.splider.setOrientation(Qt.Horizontal)
        self.__hsv_lay.addWidget(self.hsv)
        self.__hsv_lay.addWidget(self.splider)

        self.__m_v_lay.addWidget(self.color_bar)
        self.__m_v_lay.addWidget(self.hsv_widget)

        self.__middle_lay.addWidget(self.st)

    # 纯色操作台
    def pureColorOperation(self):
        self._m_r_widget = QWidget()
        self._m_r_lay = QVBoxLayout(self._m_r_widget)
        self._m_r_widget.setFixedWidth(100)

        self.operation_st.addWidget(self._m_r_widget)

        self.label_r, self.label_g, self.label_b, self.label_a = [
            QLabel("红(R)"),
            QLabel("绿(G)"),
            QLabel("蓝(B)"),
            QLabel("透(A)"),
        ]
        self.lineedit_r, self.lineedit_g, self.lineedit_b, self.lineedit_a = [
            QLineEdit(), QLineEdit(),
            QLineEdit(), QLineEdit()
        ]
        for line in [self.lineedit_r, self.lineedit_g, self.lineedit_b, self.lineedit_a]:
            line.setText("255")
        self.lineedit_a.setText("255")
        self.formLayout = QFormLayout()
        self.formLayout.addRow(self.label_r, self.lineedit_r)
        self.formLayout.addRow(self.label_g, self.lineedit_g)
        self.formLayout.addRow(self.label_b, self.lineedit_b)
        self.formLayout.addRow(self.label_a, self.lineedit_a)
        self.colorButton = QPushButton("获取颜色")
        self.colorButton.setFixedSize(90, 30)
        self.colorButton.setObjectName("colorButton")
        self.formLayout.setWidget(self.formLayout.rowCount(), QFormLayout.SpanningRole, self.colorButton)

        self._m_r_lay.addLayout(self.formLayout)

        self.__middle_lay.addWidget(self.operation_st)

    # 线性,径向,辐射面板
    def grabColorWidget(self,g_type):
        if g_type == Handle_Linear:
            self.__line_vlay = QVBoxLayout(self.st_linear_widget)
            self.linearGradient_widget = GradientWidget(Handle_Linear)
            self.linearOperation_color = ColorOperation(Handle_Linear)
            vlay = self.__line_vlay
            gradient_widget = self.linearGradient_widget
            operation_color = self.linearOperation_color
        elif g_type == Handle_Radial:
            self.__radial_vlay = QVBoxLayout(self.st_radial_widget)
            self.radialGradient_widget = GradientWidget(Handle_Radial)
            self.radialOperation_color = ColorOperation(Handle_Radial)
            vlay = self.__radial_vlay
            gradient_widget = self.radialGradient_widget
            operation_color = self.radialOperation_color
        elif g_type == Handle_Conical:
            self.__conical_vlay = QVBoxLayout(self.st_conicalt_widget)
            self.conicalGradient_widget = GradientWidget(Handle_Conical)
            self.conicalOperation_color = ColorOperation(Handle_Conical)
            vlay = self.__conical_vlay
            gradient_widget = self.conicalGradient_widget
            operation_color = self.conicalOperation_color

        gradient_widget.setViewCode(self.view_code)

        operation_color.syncPosed.connect(
            lambda p_type, hand_id, color_scope: gradient_widget.syncPos(p_type, hand_id, color_scope))
        operation_color.syncColor.connect(
            lambda p_type, hand_id, color: gradient_widget.syncColor(p_type, hand_id, color))
        operation_color.syncNewColor.connect(
            lambda p_type, colorScope, color: gradient_widget.syncNewColor(p_type, colorScope, color))
        operation_color.syncDelColor.connect(
            lambda p_type, hand_id: gradient_widget.syncDelColor(p_type, hand_id))

        vlay.addWidget(gradient_widget)
        vlay.addWidget(operation_color)

    # 线性右侧操作区域
    def linearColorOperation(self):
        self._m_r_line_widget = QWidget()
        self._m_r_line_widget.setFixedWidth(100)

        # 待写
        self.linear_view = QPushButton(self._m_r_line_widget)
        self.linear_view.setObjectName("linear_view")
        self.linear_view.resize(100,24)
        self.linear_view.setText("显示代码")
        self.linear_qss = QPushButton(self._m_r_line_widget)
        self.linear_qss.resize(100,24)
        self.linear_qss.setText("QSS")
        self.linear_qss.move(5,30)

        self.operation_st.addWidget(self._m_r_line_widget)

    # 径向右侧操作区域
    def radialColorOperation(self):
        self._m_r_radial_widget = QWidget()
        self._m_r_radial_widget.setFixedWidth(100)

        self.radial_view = QPushButton(self._m_r_radial_widget)
        self.radial_view.setObjectName("radial_view")
        self.radial_view.resize(100, 24)
        self.radial_view.setText("显示代码")

        self.operation_st.addWidget(self._m_r_radial_widget)

    # 径向右侧操作区域
    def conicalColorOperation(self):
        self._m_r_conical_widget = QWidget()
        self._m_r_conical_widget.setFixedWidth(100)

        self.conical_view = QPushButton(self._m_r_conical_widget)
        self.conical_view.setObjectName("conical_view")
        self.conical_view.resize(100, 24)
        self.conical_view.setText("显示代码")

        self.operation_st.addWidget(self._m_r_conical_widget)

    def setLabelRGB(self, c: QColor):
        r, g, b, a = c.getRgb()
        self.lineedit_r.setText(str(r))
        self.lineedit_g.setText(str(g))
        self.lineedit_b.setText(str(b))

    def setDemonstrationColor(self, color: QColor):
        self.demonstration_color_l.setStyleSheet('''
        border-radius:3px;
        background-color:rgba({},{},{},{});
        '''.format(*color.getRgb()))

    def __update_color_event(self, c):
        self.setLabelRGB(c)
        self.hex_line.setText(c.name())
        self.setDemonstrationColor(c)
        self.update()

    # Updates the color block rgb event
    def __update_rgb_event(self, c: QColor):
        self.color_bar.setBgColor(QColor(c))
        self.setLabelRGB(self.color_bar.curColor())
        self.setDemonstrationColor(self.color_bar.curColor())
        self.update()

    # Update transparency
    def update_a_event(self, value: int):
        self.color_bar.setAlpha(value)
        self.lineedit_a.setText(str(value))
        self.setDemonstrationColor(self.color_bar.curColor())
        self.update()

    # emit
    def __emit_color_event(self):
        self.clickColor.emit(QColor(self.hex_line.text()))

    def straw_event(self):
        self.maskWidget = MaskWidget(self.timer)
        # size = desktopAllSize()
        # self.maskWidget.move(-size.width()//2,0)
        # self.maskWidget.resize(size)
        self.maskWidget.resize(100, 100)
        self.maskWidget.show()
        self.timer.start(50)

    def myEvent(self):
        def change_btn_event(index):
            self.st.setCurrentIndex(index)
            self.operation_st.setCurrentIndex(index)

        self.pure_color_btn.clicked.connect(lambda: change_btn_event(0))
        self.line_btn.clicked.connect(lambda: change_btn_event(1))
        self.radial_btn.clicked.connect(lambda: change_btn_event(2))
        self.conical_btn.clicked.connect(lambda: change_btn_event(3))

        # mode change
        def change_mode(mode):
            self.linearGradient_widget.setGrabMode(mode)
            self.radialGradient_widget.setGrabMode(mode)
            self.conicalGradient_widget.setGrabMode(mode)

        self.pad_btn.clicked.connect(lambda: change_mode("pad"))
        self.rep_btn.clicked.connect(lambda: change_mode("repeat"))
        self.ref_btn.clicked.connect(lambda: change_mode("reflect"))

        #
        def hide_hand_event(b):
            self.linearGradient_widget.setHideHand(b)
            self.radialGradient_widget.setHideHand(b)
            self.conicalGradient_widget.setHideHand(b)

        self.hand_btn.clicked.connect(hide_hand_event)

        # view
        def __view():
            self.view_code.show()

        self.linear_view.clicked.connect(__view)
        self.radial_view.clicked.connect(__view)
        self.conical_view.clicked.connect(__view)

        self.hsv.rgbaChange.connect(self.__update_rgb_event)
        self.color_bar.rgbaChange.connect(self.__update_color_event)

        self.splider.valueChanged.connect(self.update_a_event)

        self.colorButton.clicked.connect(self.__emit_color_event)

        self.color_straw_btn.clicked.connect(self.straw_event)


if __name__ == '__main__':
    app = QApplication(sys.argv)
    win = PaletteTools()
    # win = ColorBar()
    win.show()

    if PYQT_VERSIONS in ["PyQt6", "PySide6"]:
        sys.exit(app.exec())
    else:
        sys.exit(app.exec_())



# -*- coding: utf-8 -*-

# self implementation generated from reading ui file 'newListTemplateWindowUI.ui'
#
# Created by: PyQt5 UI code generator 5.15.4
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQtGuiLib.header import (
    QWidget,
    QHBoxLayout,
    QVBoxLayout,
    QListWidget,
    QSize,
    QPushButton,
    qt,
    Qt,
    QGridLayout,
    QIcon,
    QSizePolicy,
    QBitmap,
    QColor,
    QPainter,
    QFrame
)
from PyQtGuiLib.core import SlideShow
from PyQtGuiLib.styles import QssStyleAnalysis


class CoreWidget(QFrame):
    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)

        # 悬浮标准
        self.__suspension = False

    # 设置悬浮
    def setSuspension(self,b:bool):
        self.__suspension = b

    def paintEvent(self, e) -> None:
        if self.__suspension:
            mask = QBitmap(self.size())
            mask.fill(Qt.black)
            painter = QPainter(mask)
            painter.setRenderHints(qt.Antialiasing | qt.SmoothPixmapTransform | qt.TextAntialiasing)
            painter.setBrush(QColor(Qt.white))
            painter.drawEllipse(self.width()-98, -6, 110, 105)
            self.setMask(mask)
            painter.end()
        else:
            mask = QBitmap(self.size())
            mask.fill(Qt.black)
            self.setMask(mask)



class ListTemplateWindowUI(QWidget):
    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)

        self.setWindowTitle("ListTemplateWindow")
        self.setupUi()
        self.defaultStyle()

    def setupUi(self):
        self.resize(1280, 795)
        self.gridLayout = QGridLayout(self)
        self.gridLayout.setContentsMargins(1, 1, 1, 1)
        self.gridLayout.setSpacing(0)
        self.gridLayout.setObjectName("gridLayout")
        self.listWidget = QListWidget(self)
        self.listWidget.setMaximumSize(QSize(260, 16777215))
        self.listWidget.setObjectName("listWidget")
        self.gridLayout.addWidget(self.listWidget, 0, 0, 2, 1)
        self.widget_head = QWidget(self)
        self.widget_head.setMinimumSize(QSize(0, 130))
        self.widget_head.setMaximumSize(QSize(16777215, 130))
        self.widget_head.setObjectName("widget_2")
        self.horizontalLayout = QHBoxLayout(self.widget_head)
        self.horizontalLayout.setContentsMargins(5, 0, 5, 0)
        self.horizontalLayout.setSpacing(0)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.btn_fold = QPushButton(self.widget_head)
        self.btn_fold.setText("三")
        self.btn_fold.setMinimumSize(QSize(50, 50))
        self.btn_fold.setMaximumSize(QSize(50, 50))
        self.btn_fold.setStyleSheet("background-color: rgb(234, 234, 234);\n"
"border:none;\n"
"font: 14pt \"黑体\";")
        self.btn_fold.setObjectName("btn_fold")
        self.horizontalLayout.addWidget(self.btn_fold)
        self.head_middle_widget = QWidget(self.widget_head)
        self.head_middle_vbody = QVBoxLayout(self.head_middle_widget)
        self.head_middle_vbody.setContentsMargins(0,0,0,0)
        self.head_middle_vbody.setSpacing(0)
        self.head_middle_widget.setObjectName("head_middle_widget")
        self.horizontalLayout.addWidget(self.head_middle_widget)
        self.btn_head_picture = QPushButton()
        self.btn_head_picture.setMinimumSize(QSize(100, 100))
        self.btn_head_picture.setMaximumSize(QSize(100, 100))
        self.btn_head_picture.setObjectName("btn_head_picture")
        self.horizontalLayout.addWidget(self.btn_head_picture)
        self.gridLayout.addWidget(self.widget_head, 0, 1, 1, 1)
        self.core_widget = CoreWidget(self)
        self.core_widget.setObjectName("core_widget")
        self.core_widget.setSizePolicy(QSizePolicy.Expanding,QSizePolicy.Expanding)
        self.gridLayout.addWidget(self.core_widget, 1, 1, 1, 1)
        self.core_vboy = QVBoxLayout(self.core_widget)
        self.core_vboy.setContentsMargins(6,6,6,6)
        self.stackedWidget = SlideShow()
        self.stackedWidget.setButtonsHide(True)
        self.core_vboy.addWidget(self.stackedWidget)

        self.listWidget.setVerticalScrollBarPolicy(qt.ScrollBarAlwaysOff)
        self.listWidget.setHorizontalScrollBarPolicy(qt.ScrollBarAlwaysOff)

    def defaultStyle(self):
        self.setStyleSheet('''
        background-color: #fff;
        border:none;
        ''')

        self.qss_head_picture = QssStyleAnalysis(self.btn_head_picture)
        self.qss_head_picture.setQSS(r'''
        QPushButton{
        border:2px solid #789ac9;
        border-radius:50px;
        }
        QPushButton:hover{
        border:2px solid #f7e151;
        }
        ''')
        self.qss_head_picture.selector("QPushButton").updateAttr("border-radius","{}px".format(self.btn_head_picture.width()//2))

        self.qss_listwiget = QssStyleAnalysis(self.listWidget)
        self.qss_listwiget.setQSS('''
QListView {
border-right:1px solid rgb(234,234,234);
outline: 0px;
background-color: #fff;
color: #000;
font: 14pt "黑体";
}
QListView::item{
padding: 5px;
}
QListView::item:hover{
background-color: rgba(170, 255, 255,100);
color: #000;
}
QListView::item:selected{
border-right:5px solid #0055ff;
background-color: rgba(170, 255, 255,200);
color: #000;
}
        ''')



from PyQtGuiLib.header import (
    QWidget,
    QMessageBox
)

from PyQtGuiLib.core.comboBox.comboCheckBox.py_style import PyStyle


class Py_Message_box(QMessageBox):
    def __init__(self,
                 parent: QWidget = None,
                 obj_name=None,
                 title=None,
                 icon=None,
                 width=390,
                 height=120,
                 style=None,
                 message_txt=None,
                 yes_txt=None,
                 no_txt=None,
                 type=None,
                 is_frame=True,
                 ):
        super(Py_Message_box, self).__init__()

        self._parent=parent
        self._obj_name=obj_name
        self._title=title
        self._icon =icon
        self._width =width
        self._height=height
        self._style=style
        self._message_txt=message_txt
        self._yes_txt=yes_txt
        self._no_txt=no_txt
        self._type=type
        self._isFrame = is_frame

        # 设置名称
        if self._obj_name is not None:
            self.setObjectName(self._obj_name)

        # 设置标题
        if self._title is not None:
            self.setWindowTitle(self._title)
        # 设置按钮的尺寸范围
        if self._width is None:
            if self._height is None:
                pass
            else:
                self.setMinimumHeight(self._height)
                self.setMaximumHeight(self._height)
        else:
            if self._height is None:
                self.setMinimumWidth(self._width)
                self.setMaximumWidth(self._width)
            else:
                self.setFixedSize(self._width, self._height)
        if type=='information':
            self.informat(self._message_txt)

        self.set_styleSheet(self._style)
        self.exec_()
        self.set_styleSheet(self._style)


    def set_styleSheet(self, style=None):
        """设置样式"""
        if style is not None:
            self.setStyleSheet(style)
        else:
            self.setStyleSheet(PyStyle.messagebox)

    def informat(self,txt):
        self.setText(txt)
        self.setStandardButtons(QMessageBox.Yes)

        buttonY = self.button(QMessageBox.Yes)
        buttonY.setText('确 定')





from PyQtGuiLib.header.versions import PYQT_VERSIONS

if PYQT_VERSIONS == "PyQt5":
    from PyQt5.QtWidgets import *
    from PyQt5.QtWidgets import QDesktopWidget as DesktopWidget

if PYQT_VERSIONS == "PyQt6":
    from PyQt6.QtWidgets import *

if PYQT_VERSIONS == "PySide2":
    from PySide2.QtWidgets import *
    from PySide2.QtWidgets import QDesktopWidget as DesktopWidget

if PYQT_VERSIONS == "PySide6":
    from PySide6.QtWidgets import *


# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'qssFile.py'
#
# Created by: PyQt5 UI code generator 5.15.4
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.





from PyQtGuiLib.header import (
    QParallelAnimationGroup,
    QSequentialAnimationGroup,
    QPushButton,
    QObject,
    qt,
    QRect,
    QColor
)
from typing import TypeVar,List
from collections.abc import Callable
import copy
'''
    重构动画框架
    Animation 仅创建,管理,播放动画
'''
from abandonCase.animation.animationFactory import AnimationFactory,PropertyAnimation
from abandonCase.animation.animationDrawType import AniNumber,AniNumbers,AniColor,AniRect,AniShadow,AniPoint

AniMode = int
ObjMode = str
AniStartType = TypeVar("AniStartType",AniNumber,AniNumbers,AniColor,AniRect)
AniEndType = TypeVar("AniEndType",list,QColor,QRect)
AnimationModeType = TypeVar("AnimationModeType",QParallelAnimationGroup,QSequentialAnimationGroup)


# 动画属性类
class AnimationAttr:
    '''

        并行模式
        串行模式
        混合模式
    '''
    Parallel = 1
    Sequential = 2

    # 动画对象模式
    Control = "control"  # 动画作用在普通控件上面
    Draw = "draw"        # 动画作用在绘制的图形上面

    # 动效
    InCurve = qt.InCurve
    OutBounce = qt.OutBounce
    CosineCurve = qt.CosineCurve
    SineCurve = qt.SineCurve

    def __init__(self,parent:QObject=None,ani_obj_mode="control"):
        self.__parent = None
        self.__ani_obj_mode = ani_obj_mode
        self.__ani_group_obj = None # 动画组对象

        # 默认动画模式(并行)
        self.__ani_mode = AnimationAttr.Parallel
        # 默认动画时长
        self.__ani_duration = 1000
        # 默认动效
        self.__ani_special = AnimationAttr.InCurve
        # 默认动画循环次数
        self.__ani_loopCount = 1

        if parent:
            self.setParent(parent)

    def setParent(self,parent):
        self.__parent = parent

    def setAniMode(self,mode:AniMode):
        self.__ani_mode = mode

    def setDuration(self,duration:int):
        self.__ani_duration = duration

    def setSpecial(self, special):
        self.__ani_special = special

    def setLoopCount(self, count:int):
        self.__ani_loopCount = count

    def setAniGroupObj(self,obj:AnimationModeType):
        if isinstance(obj,QParallelAnimationGroup) or isinstance(obj,QSequentialAnimationGroup):
            self.__ani_group_obj = obj
        else:
            raise Exception("Attribute error!")

    def parent(self) -> QObject:
        return self.__parent

    def aniMode(self) -> AniMode:
        return self.__ani_mode

    def duration(self) -> int:
        return self.__ani_duration

    def special(self):
        return self.__ani_special

    def loopCount(self) -> int:
        return self.__ani_loopCount

    def aniObjMode(self) -> ObjMode:
        return self.__ani_obj_mode

    def aniGroupObj(self) -> AnimationModeType:
        return self.__ani_group_obj

    def isAniGroupObj(self) -> bool:
        return True if self.aniGroupObj() else False

    def isControlMode(self) -> bool:
        return True if self.aniObjMode() == AnimationAttr.Control else False

    def isDrawMode(self) -> bool:
        return True if self.aniObjMode() == AnimationAttr.Draw else False

    # 绘图动画的 值封装
    @staticmethod
    def createAniNumber(value) -> AniNumber:
        return AniNumber(value)

    # 绘图动画的 多值封装
    @staticmethod
    def createAniNumbers(*args) -> AniNumbers:
        return AniNumbers(*args)

    # 绘图动画的 颜色值封装
    @staticmethod
    def createAniColor(*args) -> AniColor:
        return AniColor(*args)

    # 绘图动画的 矩形值封装
    @staticmethod
    def createAniRect(*args) -> AniRect:
        return AniRect(*args)

    # 绘图动画的 阴影封装
    @staticmethod
    def createAniShadow(*args) -> AniShadow:
        return AniShadow(*args)

    # 点封装
    @staticmethod
    def createPoint(*args) -> AniPoint:
        return AniPoint(*args)


# 动画的基类,抽离不同动画模式的共同点
class AnimationABC(AnimationAttr):
    def __init__(self, parent: QObject = None, ani_obj_mode="control"):
        super().__init__(parent,ani_obj_mode)

        # 动画列表(里面每一个元素都是一个完整的动画对象)
        self.__ani_list = []

    def allAniList(self) -> list:
        return self.__ani_list

    def appendAni(self,ani:PropertyAnimation):
        self.allAniList().append(ani)

    # 设置通用属性
    def setGeneralAttr(self,ani_data:dict):
        duration = ani_data.get("duration", None)
        special = ani_data.get("special", None)
        loopCount = ani_data.get("loop", None)

        if duration is None:
            ani_data["duration"] = self.duration()
        if special is None:
            ani_data["special"] = self.special()
        if loopCount is None:
            ani_data["loopCount"] = self.loopCount()

    # 创建动画
    def createAni(self,ani_data:dict) -> PropertyAnimation:
        '''
         {
             "targetObj":xx
             "propertyName":""
             "duration":1000,  # 可选参数
             "special":        # 可选参数
             "loop":1          # 可选参数
             "call":fun  回调函数  # 可选参数
             "argc":tuple    # 回调函数的参数  可选参数
             "sv":xx  # 该参数在控件模式下,可以写 this 指向自己的属性
             "atv":[()] 或者 []          # 可选参数
             "ev":xx
             "selector":""  # 选择器,这个参数qss样式动画时使用 eg:backgroundColor  可选参数
             "qss-suffix":"px"  # qss属性单位 如果: 写宽度时单位是px,表示文字大小时,有时候会用到pt  可选参数
             "isEffect":True/False # 这个参数觉得了是否开启特殊动画,阴影,模糊 可选参数
             "comment":xx  # 备注  可选参数
             "blendFlag":True/False 混合动画标志,表示在混合模式下这类东西是串行的  可选参数
         }
         注意 sv,atv,ev 中的值的参数类型必须一致
         这里的每一个动画都是独立的,不会在并联动画连续播放
         普通控件动画最简化版
         {
             "targetObj":xx,
             "propertyName":"",
             "sv":xx,
             "ev":xx
         }
         QSS属性动画的最简化版
        {
             "targetObj":xx,
             "propertyName":"",
             "sv":xx,
             "ev":xx,
             "selector":xx
         }
        绘图动画最简化版
         {
             "propertyName":"",
             "sv":xx,
             "ev":xx
         }
         :param ani_data:
         :return:
         '''
        if not ani_data:
            return

        self.setGeneralAttr(ani_data)

        if self.isControlMode() and self.parent():
            ani = AnimationFactory(self.parent(), ani_data, self.aniObjMode()).createAni()
        elif self.isDrawMode():
            ani_data["targetObj"] = QObject()
            ani = AnimationFactory(self.parent(), ani_data, self.aniObjMode()).createAni()
        else:
            raise Exception("Pattern error,Only Animation.Control or Animation.Draw is supported!")
        return ani

    def addAni(self,ani_data:dict) -> None:
        if not ani_data:
            raise Exception("The first argument cannot be null!")
        self.appendAni(self.createAni(ani_data))

    def addAnis(self,*args):
        for ani_data in args:
            self.addAni(ani_data)

    # 把自身所有的子动画构建成一个并行动画
    def buildParallel(self) -> QParallelAnimationGroup:
        parallel = QParallelAnimationGroup()
        for ani in self.allAniList():
            parallel.addAnimation(ani)
        return parallel

    # 把自身所有的子动画构建成一个串行动画
    def buildSequential(self) -> QSequentialAnimationGroup:
        sequential = QSequentialAnimationGroup()
        for ani in self.allAniList():
            sequential.addAnimation(ani)
        return sequential

    def buildAni(self):
        if self.aniMode() == AnimationABC.Parallel:
            return self.buildParallel()
        if self.aniMode() == AnimationABC.Sequential:
            return self.buildSequential()


class AnimationContinuous(AnimationABC):
    '''
        连续的动画类
    '''
    def __init__(self, parent: QObject = None, ani_obj_mode="control"):
        super().__init__(parent,ani_obj_mode)

        # 默认串行
        self.setAniMode(AnimationABC.Sequential)

    # 添加连续动画(运动必须是相同的)
    def addSeriesAni(self, ani_data: dict, variation: list):
        '''
            连续动画是由单个具有相同的行为动画的组合
        :param ani_data:
        :param variation:
        :return:
        '''
        if not ani_data:
            raise Exception("The first argument cannot be null!")

        if self.aniMode() != AnimationABC.Sequential:
            raise Exception("Current animation mode is not Animation.Sequential")

        if not self.isDrawMode() and not self.isControlMode():
            raise Exception("Pattern error,Only Animation.Control or Animation.Draw is supported!")

        if not self.parent():
            raise Exception("No superclass")

        self.setGeneralAttr(ani_data)

        '''
                      如果有回调函数,保存之后,先移除掉,最后添加在末尾
                  '''
        Call = ani_data.get("call", None)
        CallAgrc = ani_data.get("argc", None)
        if Call: del ani_data["call"]
        if CallAgrc: del ani_data["argc"]

        # 首先创建出 初始动画
        ani = AnimationFactory(self.parent(), ani_data, self.aniObjMode()).createAni()
        self.appendAni(ani)
        sv = ani.endValue()

        if self.isDrawMode():
            ani_data["targetObj"] = QObject()

        # 在创建出后续的连续动画
        variation_len = len(variation)
        for i, ev in enumerate(variation):
            if i == variation_len - 1:  # 这里判断是否是最后一项数值
                if Call: ani_data["call"] = Call
                if CallAgrc: ani_data["argc"] = CallAgrc
            ani_data["sv"] = sv
            ani_data["ev"] = ev
            e_ani = AnimationFactory(self.parent(), ani_data, self.aniObjMode()).createAni()
            sv = ev
            self.appendAni(e_ani)


class AnimationValues(AnimationABC):
    def __init__(self, parent: QObject = None, ani_obj_mode="draw"):
        super().__init__(parent,ani_obj_mode)

    # 绘图-多值动画
    def addValuesAni(self,ani_data: dict,startObj:AniStartType,ends:AniEndType):
        '''


        :param ani_data:
        :param startObj: 参数类型 AniColor,AniNumbers
        :param ends:
        :return:
        '''
        if self.isDrawMode():
            # 先移除属性
            if not ani_data:
                raise Exception("The first argument cannot be null!")

            Sv = ani_data.get("sv",None)
            Ev = ani_data.get("ev",None)
            Call = ani_data.get("call",None)
            if Sv:del ani_data["sv"]
            if Ev:del ani_data["ev"]
            if Call:del ani_data["call"]

            # 类型检测,并转换
            if isinstance(ends,QColor):
                ends = ends.getRgb()
            elif isinstance(ends,QRect):
                ends = ends.getRect()

            for sv,ev in zip(startObj.numberObjs(),ends):
                if self.isDrawMode():
                    TargetObj = ani_data.get("targetObj")
                    if TargetObj: del ani_data["targetObj"]
                    copy_ani_data = copy.deepcopy(ani_data)
                else:
                    copy_ani_data = copy.copy(ani_data)
                copy_ani_data["targetObj"] = copy_ani_data.get("targetObj",QObject())
                copy_ani_data["sv"] = sv
                copy_ani_data["ev"] = ev
                self.addAni(copy_ani_data)
        else:
            raise Exception("addValuesAni() This method supports only the Animation.Draw mode!")


# 混合模式
class AnimationBlend(AnimationABC):
    def __init__(self, parent: QObject = None, ani_obj_mode="control"):
        super().__init__(parent,ani_obj_mode)

        self.setAniMode(AnimationABC.Sequential)

    def addBlend(self, ani_datas: list):
        '''
            混合模式必须的串行
            混合动画,专注处理,即需要并行的动画,由需要串行的动画
                    n个动画组成并行 --> n个动画组成串行 --> n个动画组成并行,...
            [
                {
                "targetObj": xxx,
                "propertyName": xxx,
                "sv": xxx,
                "ev": xxxx,
                },
                {
                "targetObj": xxx,
                "propertyName": xxx,
                "sv": xxx,
                "ev": xxxx,
                },
                {
                "targetObj": xxx,
                "propertyName": xxx,
                "sv": xxx,
                "ev": xxxx,
                "blendFlag":True
                },
                {
                "targetObj": xxx,
                "propertyName": xxx,
                "sv": xxx,
                "ev": xxxx,
                "blendFlag":True
                },
                ....
            ]
            这个参数 表示先执行一组 并行动画之后,在执行一组串行动画
        :return:
        '''
        if not ani_datas:
            return

        if self.aniMode() != AnimationABC.Sequential:
            raise Exception("The blend mode must be Animation.Sequential")

        # 并行/串行动画列表
        ani_list = []

        '''
            根据第一个动画数据进行动画标记
        '''
        ani_flag = AnimationABC.Sequential if ani_datas[0].get("blendFlag", False) else AnimationABC.Parallel

        temp_list = []

        for data in ani_datas:
            if data.get("blendFlag", False) is False:
                if ani_flag == Animation.Sequential:
                    ani_list.append({"datas": temp_list, "aniMode": AnimationABC.Sequential})
                    temp_list = []

                ani_flag = AnimationABC.Parallel
            else:
                if ani_flag == AnimationABC.Parallel:
                    ani_list.append({"datas": temp_list, "aniMode": Animation.Parallel})
                    temp_list = []

                ani_flag = AnimationABC.Sequential
            temp_list.append(self.createAni(data))

        if ani_flag == AnimationABC.Sequential:
            ani_list.append({"datas": temp_list, "aniMode": AnimationABC.Sequential})
        else:
            ani_list.append({"datas": temp_list, "aniMode": AnimationABC.Parallel})

        # 混合
        blend_list = []
        for data in ani_list:
            data: dict
            aniMode, datas = data["aniMode"], data["datas"]

            if aniMode == AnimationABC.Parallel:
                g_ani = QParallelAnimationGroup()
            else:
                g_ani = QSequentialAnimationGroup()

            for ani in datas:
                g_ani.addAnimation(ani)
            blend_list.append(g_ani)

        for ani in blend_list:
            self.appendAni(ani)


# 动画类
class Animation(AnimationAttr):
    '''

        主要方法:
            addAni()  最常用的添加单个动画的方式
            addAnis() 添加多个单个动画的方式
            addSeriesAni() 添加连续动画的方法
                - 连续动作必须一致,Eg: 初始动作是pos,那么后续动作也必须的pos
                - 这个方法必须最串行模式下运行

            addValuesAni() 多值动画,这个方法比较特殊
                - 使用这个方法之前,这个类的初始模式必须会绘图模式
            addBlend() 添加混合动画
                - 这个方法必须最串行模式下运行
                - 这个方法最好单独使用,不要和其他添加动画的方式一起使用
    '''

    def __init__(self, parent: QObject = None):
        super().__init__(parent,"control")

        self.__ani_build_groups = []

    def addAni(self,ani_data:dict):
        ani = AnimationABC(self.parent())
        ani.setAniMode(self.aniMode())
        ani.setDuration(self.duration())
        ani.setSpecial(self.special())
        ani.addAni(ani_data)

        self.__ani_build_groups.append(ani)

    def addAnis(self,*args):
        for ani in args:
            self.addAni(ani)

    # 添加连续动画(运动必须是相同的)
    def addSeriesAni(self, ani_data: dict, variation: list):
        ani_continuous = AnimationContinuous(self.parent())
        ani_continuous.setDuration(self.duration())
        ani_continuous.addSeriesAni(ani_data,variation)

        self.__ani_build_groups.append(ani_continuous)

    # 绘图动画
    def addValuesAni(self, ani_data: dict, startObj: AniStartType, ends: AniEndType):
        ani_vales = AnimationValues(self.parent())
        ani_vales.setDuration(self.duration())
        ani_vales.setSpecial(self.special())
        ani_vales.addValuesAni(ani_data,startObj,ends)

        self.__ani_build_groups.append(ani_vales)

    # 混合模式
    def addBlend(self, ani_datas: list):
        '''
            该模式不支持绘图动画
        :param ani_datas:
        :return:
        '''
        ani_blend = AnimationBlend(self.parent())
        ani_blend.setDuration(self.duration())
        ani_blend.addBlend(ani_datas)

        self.__ani_build_groups.append(ani_blend)

    # Executive function
    def __exeCall(self,call:Callable, argc=None):
        if call:
            if argc:
                call(*argc)
            else:
                call()

    def pause(self,call:Callable=None, argc=None):
        if self.isAniGroupObj():
            self.aniGroupObj().pause()
            self.__exeCall(call,argc)

    def resume(self,call:Callable=None,argc=None):
        if self.isAniGroupObj():
            self.aniGroupObj().resume()
            self.__exeCall(call,argc)

    def state(self):
        if self.isAniGroupObj():
            if self.aniGroupObj().state() == self.aniGroupObj().Running:
                return self.aniGroupObj().Running
            if self.aniGroupObj() == self.aniGroupObj().Paused:
                return self.aniGroupObj().Paused
        else:
            return None

    def start(self):
        if self.aniMode() == Animation.Parallel:
            self.setAniGroupObj(QParallelAnimationGroup(self.parent()))
        if self.aniMode() == Animation.Sequential:
            self.setAniGroupObj(QSequentialAnimationGroup(self.parent()))

        for ani in self.__ani_build_groups:
            self.aniGroupObj().addAnimation(ani.buildAni())

        self.aniGroupObj().setLoopCount(self.loopCount())
        self.aniGroupObj().start()

    def allAni(self) -> list:
        return self.__ani_build_groups

    # 开关
    def aniSwitch(self,btn:QPushButton=None,texts:dict=None):
        '''
            动画开关
            如果动画当前状态是运行,则暂停,反之,运行
            __________________________________
            btn: 给定一个按钮(可选)
            texts:{
                "pause":"xxx",
                "resume":"xx"
            }
        :return:
        '''
        if texts is None:
            texts = dict()

        if self.isAniGroupObj():
            pause_ = texts.get("pause", "pause")
            resume_ = texts.get("resume", "resume")

            if self.aniGroupObj().state() == self.aniGroupObj().Running:
                self.pause()
                if btn:
                    btn.setText(resume_)
            elif self.aniGroupObj().state() == self.aniGroupObj().Paused:
                self.resume()
                if btn:
                    btn.setText(pause_)

    def getAni(self,index:int) -> List[PropertyAnimation]:
        return self.__ani_build_groups[index].allAniList()

    # 根据备注获取动画
    def getCommentAni(self,comment:str) -> List[PropertyAnimation]:
        if not comment:
            return None

        infos = []
        for ani in self.__ani_build_groups:
            anis = ani.allAniList()
            for a in anis:
                if a.commentInfo() == comment:
                    infos.append(a)
        return infos

    # 更新单个动画的信息
    def updateAni(self,ani:PropertyAnimation,new_datas:dict):
        if not new_datas:
            return

        ani.updateAni(new_datas)

    # 移除一组动画
    def removeAni(self,index:int):
        del self.__ani_build_groups[index]


# -*- coding:utf-8 -*-
# @time:2023/6/116:49
# @author:LX
# @file:transmitter.py
# @software:PyCharm

from PyQtGuiLib.header import (
    PYQT_VERSIONS,
    QApplication,
    is_win_sys,
    is_mac_sys,
    sys,
    QWidget,
    QPushButton,
    QVBoxLayout,
    QHBoxLayout,
    QStackedWidget,
    QLineEdit,
    Qt,
    Signal,
    QFileDialog,
    QTextEdit,
    QDragEnterEvent,
    QDropEvent
)


class MesLineEdit(QLineEdit):
    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)
        self.setAcceptDrops(True)

    def dragEnterEvent(self, e:QDragEnterEvent) -> None:
        e.accept()

    def dropEvent(self, e:QDropEvent) -> None:
        file_path = e.mimeData().text()  # type:str
        if is_win_sys:
            file_path = file_path.replace("file:/","")
        print("event:",file_path)


class Transmitter(QWidget):
    texted = Signal(str)

    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)
        self.setFixedHeight(80)

        self.vlay = QVBoxLayout(self)
        self.top_widget = QWidget()
        self.top_widget.setStyleSheet("border:2px solid red;")
        self.top_hlay = QHBoxLayout(self.top_widget)

        self.hlay = QHBoxLayout()
        self.hlay.setAlignment(Qt.AlignLeft)
        self.hlay.setSpacing(0)

        self.emo = QPushButton()
        self.emo.setFixedSize(40,40)
        self.emo.setText("😉")
        self.emo.setObjectName("emo")
        self.line = MesLineEdit()
        self.line.setMinimumWidth(100)
        self.line.setFixedHeight(40)
        self.line.setPlaceholderText("输入消息")
        self.line.setObjectName("line")

        self.st = QStackedWidget()
        self.st.setFixedHeight(50)
        self.st.setMaximumWidth(200)
        self.ac_widget = QWidget()
        self.btn_widget = QWidget()

        self.st.addWidget(self.ac_widget)
        self.st.addWidget(self.btn_widget)

        self.hlay.addWidget(self.emo)
        self.hlay.addWidget(self.line)
        self.hlay.addWidget(self.st)

        self.vlay.addWidget(self.top_widget)
        self.vlay.addLayout(self.hlay)

        self.st.setCurrentIndex(0)
        self.acWidget()
        self.sendWidget()

        self.line.textChanged.connect(self.switchSt_event)
        self.line.returnPressed.connect(self.sendEmit)

        self.defaultStyle()

    def sendEmit(self):
        self.texted.emit(self.line.text())
        self.line.clear()

    def acWidget(self):
        self.st_hlay = QHBoxLayout(self.ac_widget)
        self.st_hlay.setContentsMargins(9,0,0,0)
        self.st_hlay.setAlignment(Qt.AlignLeft)

        self.file_dia = QPushButton()
        self.file_dia.setText("文")
        self.file_dia.setFixedSize(40,40)
        self.file_dia.setObjectName("file_dia")

        self.it_dia = QPushButton()
        self.it_dia.setText("...")
        self.it_dia.setFixedSize(40,40)
        self.it_dia.setObjectName("it_dia")

        self.st_hlay.addWidget(self.file_dia)
        self.st_hlay.addWidget(self.it_dia)

        self.file_dia.clicked.connect(self.file_event)

    def sendWidget(self):
        self.send_hlay = QHBoxLayout(self.btn_widget)
        self.send_hlay.setContentsMargins(9,0,0,0)
        self.send_hlay.setAlignment(Qt.AlignLeft)

        self.send_btn = QPushButton()
        self.send_btn.setText("发送")
        self.send_btn.setObjectName("send_btn")
        self.send_btn.setFixedSize(40,40)

        self.send_hlay.addWidget(self.send_btn)

        self.send_btn.clicked.connect(self.sendEmit)

    def switchSt_event(self,text:str):
        if text:
            self.st.setCurrentIndex(1)
        else:
            self.st.setCurrentIndex(0)

    def file_event(self):
        file_name,file_type = QFileDialog.getOpenFileName(self,"文件")
        if file_name:
            print(file_name, file_type)

    def defaultStyle(self):
        self.setStyleSheet('''
*{
font: 11pt "等线";
}
#emo,#line,#file_dia,#it_dia{
border:none;
background-color:#fff;
}
#emo{
border-top-left-radius:10%;
border-bottom-left-radius:10%;
}
#line{
border-top-right-radius:10%;
border-bottom-right-radius:10%;
}
#file_dia,#it_dia{
border-radius:10%;
}
#send_btn:hover{
background-color: rgb(65, 130, 195);
}
#send_btn,#send_btn:pressed{
border-radius:10%;
border:none;
background-color:#55aaff;
font: 10pt "等线";
}

        ''')


if __name__ == '__main__':
    app = QApplication(sys.argv)

    win = Transmitter()
    win.show()

    if PYQT_VERSIONS in ["PyQt6","PySide6"]:
        sys.exit(app.exec())
    else:
        sys.exit(app.exec_())


# -*- coding:utf-8 -*-
# @time:2022/12/128:53
# @author:LX
# @file:developmentLog.py
# @software:PyCharm
'''
    开发日志文件
'''


r'''
2022.12.13  0.0.1.4版本  气泡窗口已经基本实现,持续优化更新
2022.12.13  0.0.1.5版本  重写了气泡窗口的部分实现代码
2022.12.19  0.0.2.0版本
    --- 气泡窗口 优化了代码,修复了文字大小无法设置的BUG,以及文字位置的问题
    --- 新增无边框窗口 BorderlessWidget 类,导入方式 from PyQtGuiLib.core.widgets import BorderlessWidget
    --- 新增圆角窗口 RoundWidget 类,导入方式 from PyQtGuiLib.core.widgets import RoundWidget
    --- 靠边窗口功能完成(任意窗口都可以加上该功能),PullOver类 导入方式 from PyQtGuiLib.core.pullOver import PullOver
    --- 靠边窗口功能的测试用例已编写
2022.12.20  0.0.3.0版本
    --- 新增圆环进度条
2022.12.22  0.0.4.0版本
    --- 修复了一些小的BUG
    --- 放弃了亚克力窗口的研究
    --- 新增加载进度条
2022.12.22  0.0.5.0版本
    --- 新增水球进度条
    --- 新增进度条的集合测试
    --- 修改了气泡窗口的导入方式
         原来的导入方式: from PyQtGuiLib.core.bubbleWidget import BubbleWidget
         现在的导入方式: from PyQtGuiLib.core import BubbleWidget
    --- 修改了窗口靠边功能的导入方式
         原来的导入方式: from PyQtGuiLib.core.pullOver import PullOver
         现在的导入方式: from PyQtGuiLib.core import PullOver
2022.12.26  0.0.6.0版本
    --- 新增轮播组件,2022.12.22  已经基本完成,可能还存在BUG,以后修改(测试发布)
2022.12.26  0.0.6.1版本
    --- 新增轮播组件 新增两个方法,setAnimationTime(),setAinDirectionMode()
2022.12.26  0.0.7.0版本
    --- 新增线性渐变进度条,到目前为止,这是唯一个不提供文字显示的进度条
    --- 更新的所有进度条的抗锯齿写法 painter.setRenderHints(painter.Antialiasing | painter.SmoothPixmapTransform | painter.TextAntialiasing)
2023.1.4  0.0.8.0版本
    --- 线性渐变进度条 增加一个修改背景底色的方法 setBackGroundColor()
    --- 感谢[PyQt5学习爱好群-(讨厌自己)提供的BUG修复思路] 修复Borderless右下角拉伸BUG
    --- 新增 标题栏 TitleBar 类(测试发布中)
2023.1.4 0.0.8.1版本
    --- 修复 TitleBar 标题栏 在多个屏幕下的BUG
    --- TitleBar 在点击关闭按钮,新增隐退效果
    --- TitleBar 放大按钮动画增强
    --- TitleBar 增加缩小动画
2023.1.4 0.0.8.2版本
    --- TitleBar 更新Mac风格的放大,缩小,关闭按钮风格
    --- TitleBar 新增 setAniDuration() 方法 设置动画的时长
    --- TitleBar 新增 setTitleIcon(),setSyncWindowIcon() 与设置图标相关的方法
    --- 优化了TitleBar部分代码
2023.1.4 0.0.8.3版本
    --- 修复 TitleBar 类在mac下图标显示位置的问题
    --- utility.py 新增 系统判断
    --- 新增状态栏
    --- 状态栏 新增 setStatusPos() 方法
2023.1.5 0.0.8.3版本
    --- 优化 TitleBar,StatusBar 代码,并修复了样式表设置无效的BUG
2023.1.5 1.0.8.3版本
    --- 新增 styles 皮肤包项目 
    --- 全面兼容 PyQt5,PyQt6,PySide2,PySide6, PySide6 在使用无边框的还有小问题
2023.1.7 1.0.9.3版本
    --- 新增流式布局(FlowLayout)
    --- 互补色样式(contrastStyle())，同色调样式(homologyStyle())
    --- 修复 标题栏和状态栏 无法绘制文字的BUG
2023.1.8 1.0.9.4版本
    --- 修复流式布局 在mac的间距问题
    --- 优化 标题栏 和 状态栏 的代码
    --- 优化获取文字大小函数 textSize() 对丢失像素进行补偿
    --- 优化标题栏 在PyQt6,PySide6,标题高度计算不精准的Bug
2023.1.8 1.1.9.4版本
    --- 感谢! PyQt5学习爱好群-(讨厌自己)修复 PyQt6 版本下 无边框类(Borderless)移动BUG
    --- 增加 内置-样式设计器,主要是为了配合皮肤包
2023.1.9 1.1.9.5版本
    --- 标题栏和状态栏 终于 不用主窗口中重写 resizeEvent事件来调用 updateXXX() 来完成大小自适应了
    --- 修复了 内置-样式设计器 不少BUG
2023.1.9 1.2.10.5版本
    --- 新增 控件的组成分析函数 导入方式 from PyQtGuiLib.core.resolver import dumpStructure
    --- 新增 QListWidget 增强版本 - ListWidget
    --- 轮播组件重写中
2023.1.12 - 2023-1.13 [1.2.12.5] 版本
    --- 删除 文件buttonWidget.py 和 roundWidget.py
    --- 重构的 无边框的代码 增加了许多功能,请看文档
    --- 标题栏 新增了 窗口钉住功能, Win 和 Mac 各有一套风格
    --- 新增功能 滚动栏(RollWidget)
    --- ListWidget 新增一种动画效果
    --- 新增 说明 流式布局 无法 配合 QScrollArea 使用
2023.1.14 1.2.13.6
    --- 重新修改了标题栏的继承方式
    --- 更新 无边框窗口 的代码,修复无边框窗口在作为子窗口时,不显示的BUG,
        并失去做为主窗口时的部分功能
    --- 新增了一个 动态标题输入框(DynamicTLine)
    --- 对流式布局进行了优化
    --- 移除了 内置样式设计器的 controlConfig.json 改为 controlConfig.py
2023.1.15 1.2.15.6
    --- 新增开关按钮(SwitchButton)
    --- 新增调色版(ColorPalette)
2023.1.15 1.2.15.7
    --- 调色版相关问题
            1.修复中间区域无法获取颜色的BUG
            2.修复中间区域点击无法触发信号BUG
            3.新增一个获取当前选中颜色的按钮
            4.新增两个信号(colorNamed,clicked)
            5.新增两种风格(Style_Black,Style_White)
    --- 重构 气泡窗口
2023.1.20 [1.2.17.7]
    --- 气泡窗口 重构完成,并使用使用了全新的QSS来设置样式
    --- 轮播组件 重构完成
    --- 以及更新部分测试用例
2023.1.21 [1.2.17.8]
    --- 状态栏 无法实时跟随窗口,需要手动的调用 updateStatusSize()
# ====== 从 2.2.17.8 版本开始 所有的样式将有全新的自定义QSS来代替
2023.1.31 [2.2.17.8]
    --- 重构了整个widgets项目,将原本的widgets移动到abandonCase目
        -- 目前只有 borderlessWidget 可用,主窗口,标题栏,状态栏,还在编写中
    --- 从 2.2.17.8这个版本开始所有的样式将有全新的自定义QSS来代替,并且不兼容前面的版本
        --- 支持自定义QSS的已经在文档中标出
    --- 优化了部分控件的代码,删除了一些不常用的控件,和测试用例
    --- 修改了大部分控件的继承对象
2023.2.9 [2.3.17.8]
    --- 感谢! PyQt5学习爱好群-(迷糊蛋der～)在2.2.17.8版本在中提出的qt版本,代码上的兼容性问题
    --- 修复的pyside兼容性的问题,
    --- 新增 QSS解析器 QssStyleAnalysis 
2023.2.13 - 2023.2.15 [2.3.18.9]
    --- 修复 QSS解析器 在追加新的属性时会覆盖原来相同属性的BUG. 目前
        追加新的属性时,会与原来的相同属性进行融合
    --- 新增 功能函数 loadUic 统一四个版本加载 UI文件 的函数,对外提统一的接口
    --- 新增滑块(Slider)

2023.2.18 - 2023.2.20 [2.4.18.11]
    --- 修改原来导入导入颜色板的方式
        原来: from PyQtGuiLib.core import ColorPalette (与这个相关文件已删除)
        现在: from PyQtGuiLib.core import PaletteFrame
        并更新代码,将原来的代码拆分为,获取颜色的框架,与独立的颜色区域,
        在框架中可以嵌入不同的颜色区别,目前支持(Rect,Wheel)
    --- 修复 QSS解析器 已知BUG,
        修复 isSelectKey() 取值成功,但是值为0,导致整个函数返回False的BUG
        新增 header(),parent()方法
        并新增对 无选择器的QSS语法  
                  color: rgb(0, 255, 127);
                  background-color:rgb(0, 170, 0);
        进行处理,该语法只能用过setQSS方法设置
        新增 QSS解析器例子,位置 PyQtGuiLib -> tests -> test_QssStyleAnalysis -> eg6.py
    --- 动态样式链接器 from PyQtGuiLib.styles import StyleLinker
        目前仅支持 QPushButton,QLabel,QLineEdit 控件
        
2023.2.20 - 2023.3.23 [2.5.18.11]
    调整结构 将 abandonCasem目录,Log目录和tests目录移出项目
    新增ToolListWidget 控件(未完成)
    新增SuperPainter 超级画师(未完成) - 重构中
    新增Animation 动画框架(未完成) - 测试部分

2023.3.24 - 2023.4.7
    - Animation 动画框架 已经快接近完成,目前大部分功能已经可用
    - 修改QSS解析器的已知BUG,并且在多线程时,是安全的
    - 新增 SuperPainter 超级画师类
        - 导入方法 from PyQtGuiLib.styles import SuperPainter
        - 教学案例在test\test_SuperPainter 目录下
2023.4.7 - 2023.4.11
    - 更新模板窗口 ListTemplateWindow
2023.4.13 
     感谢! PyQt5学习爱好群-(讨厌自己) 贡献 CircularProgressBar 进度条模块
     导入方式 from PyQtGuiLib.core.progressBar import CircularProgressBar
2023.4.20
    - 新增 抽屉控件(Drawer) -- 编写中
2023.4.21 
    -抽屉控件(Drawer)完成
2023.4.25
    -修复PySide2导入pyqtProperty的BUG
    -重构了动画框架,并删除老代码以及老的测试用例,新的动画案例还未编写
    -删除styles包下整个linker项目,以及测试用例
    -所有由其他人贡献的功能模块里面都会有一个author()方法来记录作者信息
    -新增控件ComboCheckBox,该控件是一个具体提示的输入框,同时也是一个多项选择框,
        控件由 "PyQt5学习爱好群-知行合一" 贡献
        该控件目前PySide系列支持性比较好,PyQt系列存在提示BUG
        导入方式 from PyQtGuiLib.core import ComboCheckBox
    -删除老版本的超级画师代码,重构版本的超级画师目前代码提示还不完善
    -目前所有文档还未更新,
2023.4.26
    新增控件 PageCuttingButtonGroup 分页组件
        控件由 "PyQt5学习爱好群-(讨厌自己)" 贡献
        导入方式 from PyQtGuiLib.core import PageCuttingButtonGroup
2023.4.28
    - 修改协议 MIT -> Apache License 2.0
2023.5.2
    - 修复Drawer抽屉控件,添加QListWidget的BUG
'''


# -------------------------------
'''
    暂时搁浅的任务
1.新无边框窗口主窗口,(还没有设计完成,暂时先放下 2023.1.31)
'''

# ===============================================

'''
记录已知BUG,但是还未修复
2023.2.18 
    LoadBar(加载进度条) -- 有BUG,窗口变化时,进度条不会跟随变化

2023.3.23 
        动画框架---
        出现BUG,当动画同时操作一个控件,不同样式状态下相同属性时出现BUG
'''


from PyQtGuiLib.header import (
    PYQT_VERSIONS,
    QApplication,
    sys,
    QWidget,
    qt,
    QPushButton,
    Qt,
    QPalette,
    QColor,
    QPaintEvent,
    QPainter,
    QBrush,
    QSize,
    QMainWindow,
    QLineEdit,
    desktopCenter,
    QPixmap,
    QLinearGradient,
    QLabel,
    QFont,
    textSize,
    QFontMetricsF,
    Qt,
    QVBoxLayout,
    QThread
)

from PyQtGuiLib.core import Slider
'''
    测试 滑块
'''
class TestTh(QThread):
    def __init__(self,*args,**kwargs):
        super(TestTh, self).__init__(*args,**kwargs)

    def run(self) -> None:
        while True:
            pass



class Test(QWidget):
    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)
        self.resize(600,600)

        self.sl = Slider(self)
        # self.sl.resize(300,40)
        self.sl.setFixedHeight(40)

        self.sl.setHoverIcon(False)
        self.sl.setMaxValue(30000)
        self.sl.setBuffValue(18000)
        self.sl.move(100,100)
        self.sl.valueChanged.connect(self.test)
        # self.sl.setBuffValue(30)
        self.sl.setStyleSheet('''
        Slider{
qproperty-radius:5;
qproperty-flowColor:rgb(255,0,0);
qproperty-iconSize:15;
qproperty-bgHeight:8;
        }
            ''')

        self.vboy = QVBoxLayout(self)
        self.vboy.addWidget(self.sl)

    def test(self,v):
        print("v:",v)

if __name__ == '__main__':
    app = QApplication(sys.argv)

    win = Test()
    win.show()

    if PYQT_VERSIONS in ["PyQt6","PySide6"]:
        sys.exit(app.exec())
    else:
        sys.exit(app.exec_())


# -*- coding:utf-8 -*-
# @time:2023/1/10 12:41
# @author:LX
# @file:SlideShow.py
# @software:PyCharm

from PyQtGuiLib.header import (
    QWidget,
    QPushButton,
    QPoint,
    QPropertyAnimation,
    Signal,
    QSize
)

'''
    组件轮播 2023.3.7 当前控件的所有功能均处于稳定状态
    SlideShow 独立版本 3.0
'''
class SlideShow(QWidget):
    # 切换窗口事件
    changeWidget = Signal(QWidget)

    # 动画的方向模式
    Ani_Left = "left"
    Ani_Right = "right"
    Ani_Down = "down"
    Ani_Up = "up"

    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)
        self.resize(700, 300)
        #
        self.widgets = []
        self._temp_widgets = [] # 临时窗口

        self.animation_time = 800 # 毫秒

        # 动画运行方向,当前动作
        self.ani_direction = (SlideShow.Ani_Left, SlideShow.Ani_Right)

        # 设置自动轮播,自动轮播的方向
        self.is_auto_slide = False
        self.auto_ani_direction = "R"

        # 窗口索引
        self.index = 0

        # 指向两个当前窗口
        self.cursor_widget_p = None # type:QWidget
        self.cursor_widget_p2 = None # type:QWidget

        # 创建两侧按钮
        self.createButtons()

    def setDuration(self,p_int):
        self.animation_time = p_int

    # 设置自动轮播
    def setAutoSlideShow(self,b:bool,interval=1500,direction_:str="R"):
        if direction_ not in ["R","L"]:
            raise TypeError("Parameter error, Support only 'R' or 'L' !")

        if interval <= self.animation_time:
            raise ValueError("The interval between auto wheel casting must be longer than the animation time!")

        self.is_auto_slide = b
        self.startTimer(interval)
        self.auto_ani_direction = direction_

    # 左上方向
    def lu_direction(self) -> str:
        return self.ani_direction[0]

    # 右下方向
    def rd_direction(self) -> str:
        return self.ani_direction[1]

    # 创建两侧按钮
    def createButtons(self):
        # 左右按钮
        self.Ani_Left_btn = QPushButton("左",self)
        self.Ani_Right_btn = QPushButton("右",self)
        self.Ani_Left_btn.setObjectName("Ani_Left_btn")
        self.Ani_Right_btn.setObjectName("Ani_Right_btn")
        self.Ani_Left_btn.resize(50,50)
        self.Ani_Right_btn.resize(50,50)
        self.Ani_Left_btn.clicked.connect(lambda :self.moveWidget("L"))
        self.Ani_Right_btn.clicked.connect(lambda :self.moveWidget("R"))
        self.btnStyle()
        self.btnPos()

    # 按钮样式
    def btnStyle(self):
        self.Ani_Left_btn.setStyleSheet('''
#Ani_Left_btn{
background-color:transparent;
border:2px solid #73ffcc;
color:#73ffcc;
border-radius:5px;
}
#Ani_Left_btn:hover{
border:2px solid #00557f;
}
        ''')
        self.Ani_Right_btn.setStyleSheet('''
#Ani_Right_btn{
background-color:transparent;
border:2px solid #73ffcc;
color:#73ffcc;
border-radius:5px;
}
#Ani_Right_btn:hover{
border:2px solid #00557f;
}
                ''')

    def btnPos(self):
        self.Ani_Left_btn.move(5, self.height() // 2 - self.Ani_Left_btn.height() // 2)
        self.Ani_Right_btn.move(self.width() - self.Ani_Right_btn.width() - 5,
                            self.height() // 2 - self.Ani_Left_btn.height() // 2)

    def count(self) -> int:
        return len(self.widgets)

    # 移动窗口
    def moveWidget(self,direction_:str):
        direction = None

        w_1 = self.widgets[self.index]
        if direction_ == "R": # self.index < self.count() - 1
            direction = self.rd_direction()
            self.index += 1
            if self.index > self.count()-1:
                self.index = 0

        if direction_ == "L": # and self.index >= 0
            direction = self.lu_direction()
            self.index -= 1
            if self.index < -self.count():
                self.index = -1

        if not direction:
            return

        w_2 = self.widgets[self.index]
        self.cursor_widget_p = w_2 # 保存当前窗口
        self.cursor_widget_p2 = w_1
        self.changeWidget.emit(w_2)# 切换窗口时发送信号
        w_1.show()
        w_2.show()

        self.ani_(w_1, w_2, direction)

    def addWidget(self,widget:QWidget):
        widget.setParent(self)

        widget.resize(self.size())
        widget.lower()
        self.widgets.append(widget)

        if self.count() > 1:
            widget.hide()
        else:
            self.cursor_widget_p = widget

    def insertWidget(self,index:int,widget:QWidget):
        widget.setParent(self)

        widget.resize(self.size())
        widget.lower()
        self.widgets.insert(index,widget)
        if index == self.getCursorWidgetIndex():
            self.cursor_widget_p = widget
            self.cursor_widget_p.show()
            if self.cursor_widget_p2:
                self.cursor_widget_p2 = self.getCursorWidget()
                self.cursor_widget_p2.hide()
                self.update()
        else:
            widget.hide()

    # 设置动作模式
    def setAinDirectionMode(self, mode:tuple):
        if mode[0] not in [SlideShow.Ani_Up,SlideShow.Ani_Left] and\
            mode[1] not in [SlideShow.Ani_Right,SlideShow.Ani_Down]:
            raise TypeError("Parameter error!")

        self.ani_direction = mode

    # 设置按钮的隐藏和显示
    def setButtonsHide(self,b:bool):
        self.Ani_Left_btn.setHidden(b)
        self.Ani_Right_btn.setHidden(b)

    def aniDirection(self) -> tuple:
        return self.ani_direction

    # 移除窗口
    def removeWidget(self,widget:QWidget):
        widget.hide()
        if widget == self.cursor_widget_p:
            index = self.widgets.index(widget)
            if index == 0 and self.count() >=1:
                wid = self.widgets[index+1]
                wid.resize(self.size())
                wid.show()
                self.cursor_widget_p = wid
            else:
                self.cursor_widget_p = None
        if widget == self.cursor_widget_p2:
            self.cursor_widget_p2 = None
        self.widgets.remove(widget)
        del widget

    def replaceWidget(self,old_widget:QWidget,new_widget:QWidget):
        if old_widget == self.cursor_widget_p:
            pass
        elif old_widget == self.cursor_widget_p2:
            pass
        else:
            pass

    # 弹出一个完全独立的窗口
    def popWidget(self,widget:QWidget):
        if widget == self.cursor_widget_p:
            index = self.widgets.index(widget)
            if index == 0 and self.count() >=2:
                wid = self.widgets[index+1]
                wid.resize(self.size())
                wid.show()
                self.cursor_widget_p = wid
            else:
                self.cursor_widget_p = None
        if widget == self.cursor_widget_p2:
            self.cursor_widget_p2 = None

        independence_widget = self.widgets.pop(self.widgets.index(widget)) # type:QWidget
        independence_widget.setParent(None)
        independence_widget.resize(300,300)
        return independence_widget

    # 切换到指定窗口
    def setCurrentIndex(self,index:int=0):
        n = 0
        direction = "R"
        if index > self.index:
            n = index - self.index

        if index < self.index:
            direction = "L"
            n = self.index - index

        for _ in range(abs(n)):
            self.moveWidget(direction)

    def ani_(self,widget_out:QWidget,widget_show:QWidget,direction:str):
        '''
            widget_out:被移走的窗口
            widget_show:准备显示的窗口
        '''

        if not widget_out.parent():
            widget_out.setParent(self)

        if not widget_show.parent():
            widget_show.setParent(self)

        wid1,wid2 = widget_out,widget_show
        wid1.resize(self.size())
        wid2.resize(self.size())


        ani_1 = QPropertyAnimation(wid1,b"pos",self)
        ani_2 = QPropertyAnimation(wid2,b"pos",self)
        ani_1.setDuration(self.animation_time)
        ani_2.setDuration(self.animation_time)

        if direction == SlideShow.Ani_Left:
            s_ani_1, e_ani_1 = QPoint(0, 0), QPoint(-self.width(), 0)
            s_ani_2, e_ani_2 = QPoint(self.width(), 0), QPoint(0, 0)
        elif direction == SlideShow.Ani_Up:
            s_ani_1, e_ani_1 = QPoint(0, 0), QPoint(0, -self.height())
            s_ani_2, e_ani_2 = QPoint(0, self.height()), QPoint(0, 0)
        elif direction == SlideShow.Ani_Down:
            s_ani_1, e_ani_1 = QPoint(0, 0), QPoint(0, self.height())
            s_ani_2, e_ani_2 = QPoint(0, -self.height()), QPoint(0, 0)
        else:
            s_ani_1,e_ani_1 = QPoint(0,0),QPoint(self.width(),0)
            s_ani_2,e_ani_2 = QPoint(-self.width(),0),QPoint(0,0)

        # 在移出窗口动画完成时,执行隐藏
        ani_1.finished.connect(lambda :wid1.hide())

        ani_1.setStartValue(s_ani_1)
        ani_1.setEndValue(e_ani_1)

        ani_2.setStartValue(s_ani_2)
        ani_2.setEndValue(e_ani_2)

        ani_1.start()
        ani_2.start()

    # 通过索引获取窗口
    def getWidget(self,index:int) -> QWidget:
        return self.widgets[index]

    # 反按钮对象
    def getButtons(self)->tuple:
        return self.Ani_Left_btn,self.Ani_Right_btn

    def isEmpty(self) -> bool:
        return True if not self.widgets else False

    def timerEvent(self, event) -> None:
        if self.is_auto_slide:
            self.moveWidget(self.auto_ani_direction)

    # 返回当前的窗口
    def getCursorWidget(self)->QWidget:
        return self.cursor_widget_p

    # 返回当前的窗口索引
    def getCursorWidgetIndex(self)->int:
        print("--->",self.getCursorWidget())
        return self.widgets.index(self.getCursorWidget())

    def resizeEvent(self, event) -> None:
        self.btnPos()
        if self.cursor_widget_p:
            self.cursor_widget_p.resize(self.size())
            # 窗口大小改变时只显示当前窗口,隐藏其他窗口
            if self.cursor_widget_p2 and self.cursor_widget_p2.isHidden() is False:
                self.cursor_widget_p2.hide()

        super().resizeEvent(event)



# -*- coding:utf-8 -*-
# @time:2023/4/717:05
# @author:LX
# @file:eg1.py
# @software:PyCharm
from PyQtGuiLib.header import (
    PYQT_VERSIONS,
    QApplication,
    sys,
    QWidget,
    QColor
)
from PyQtGuiLib.styles import SuperPainter



class Test(QWidget):
    def __init__(self):
        super().__init__()
        '''
            SuperPainter第一个案例, 简单演示使用方法,你会发现和QPainter没什么区别
        '''
        self.painter = SuperPainter() # 创建一个画师对象,注意这个对象不要创建在paintEvent里面

    def paintEvent(self, QPaintEvent):
        self.painter.begin(self)

        # 首先画一个没有任何样式的矩形
        self.painter.drawRect(20,20,50,50)

        # 设置一个画笔,在画两个矩形,这里我们可以看到两个红色边框的画笔
        self.painter.setPen(QColor("red"))
        self.painter.drawRect(100, 20, 50, 50)
        self.painter.drawRect(200, 20, 50, 50)

        # 接下来我们画一个,绿色和蓝色的矩形
        self.painter.setPen(QColor("green"))  # 重新将画笔设置为绿色
        self.painter.drawRect(20, 80, 50, 50)
        self.painter.setPen(QColor("blue"))  # 再次将画笔设置为蓝色
        self.painter.drawRect(100, 80, 50, 50)

        self.painter.end()

if __name__ == '__main__':
    app = QApplication(sys.argv)

    win = Test()
    win.show()

    if PYQT_VERSIONS in ["PyQt6","PySide6"]:
        sys.exit(app.exec())
    else:
        sys.exit(app.exec_())



# -*- coding:utf-8 -*-
# @time:2023/3/716:00
# @author:LX
# @file:__init__.py.py
# @software:PyCharm
def info(f):
    def wapper(*args,**kwargs):
        print("标记")
        f(*args,**kwargs)
    return wapper


def add_info(cls):
    for key,v in cls.__dict__.items():
        if "test" in key and callable(v):
            f = info(v)
            setattr(cls,key,f)
    return cls


@add_info
class A:
    def __init__(self):
        pass

    def test_aa(self):
        print("test")

    def test2_bb(self):
        print("test2")


a = A()
a.test_aa()
a.test2_bb()


# -*- coding:utf-8 -*-
# @time:2023/5/3117:55
# @author:LX
# @file:chatRoom.py
# @software:PyCharm
from PyQtGuiLib.header import (
    QWidget,
    QScrollArea,
    QVBoxLayout,
    Qt
)

from PyQtGuiLib.core.charRooms.transmitter import Transmitter
from PyQtGuiLib.core.charRooms.message import (
    MessageBody,BGTemplate,Message,
    Left,Right,Type_Text,Type_Image
)

'''
    聊天对话框组件
'''


class ChatRoom(QScrollArea):
    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)
        self.resize(500,500)
        self.setWindowTitle("ChatRoom")

        self.body = QWidget()
        self.vlay = QVBoxLayout(self.body)
        self.vlay.setAlignment(Qt.AlignTop)
        self.vlay.setContentsMargins(0,0,0,0)
        self.vlay.setSpacing(0)

        self.body.setStyleSheet("background-color:#fff;")

        self.setWidget(self.body)
        self.setWidgetResizable(True)
        self.setVerticalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        self.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)

        self.defaultStyle()

    def addWidget(self,widget:QWidget):
        self.vlay.addWidget(widget)

    def createBGWidget(self,direction,mes:Message):
        widget = BGTemplate(self,direction,mes)
        self.addWidget(widget)
        self.scrollBottom(widget.height())

    # 滚动条置底
    def scrollBottom(self,h=140):
        dif = self.verticalScrollBar().height()-self.body.height()
        dif = abs(dif)
        if dif:
            self.verticalScrollBar().setMaximum(dif+h)
            self.verticalScrollBar().setValue(dif+h)

    def sendText(self,mes:Message) -> bool:
        if len(mes.data()) > mes.getMaxMesNum():
            return False
        self.createBGWidget(Right, mes)
        return True

    def receiveText(self,mes:Message) -> bool:
        if len(mes.data()) > mes.getMaxMesNum():
            return False
        self.createBGWidget(Left, mes)
        return True

    def resizeEvent(self, e) -> None:
        self.scrollBottom()
        super().resizeEvent(e)

    def defaultStyle(self):
        self.setStyleSheet('''
QScrollBar:horizontal{
padding:0px;
max-height:12px;
padding:2px;
}
QScrollBar:vertical{
padding:0px;
max-width:10px;
padding:2px;
}
QScrollBar::handle{
background-color: rgb(39, 39, 39);
border-radius:2px;
}
QScrollBar::handle:hover{
background-color: rgb(72, 72, 72);
border:1px solid rgb(149, 149, 149);
}
QScrollBar::add-page,QScrollBar::sub-page {
background:rgb(57, 57, 57);
}
QScrollBar::sub-line,QScrollBar::add-line {
background:none;
}
        ''')



import sys
from PyQt5.QtGui import QPainter, QPen
from PyQt5.QtCore import Qt, QPoint
from PyQt5.QtWidgets import QWidget, QApplication

class Example(QWidget):

    def __init__(self):
        super().__init__()

        self.initUI()


    def initUI(self):

        self.setGeometry(300, 300, 350, 100)
        self.setWindowTitle('Lines')

        self.show()


    def paintEvent(self, e):

        qp = QPainter()
        qp.begin(self)
        self.drawLines(qp)
        qp.end()


    def drawLines(self, qp):

        pen = QPen(Qt.black, 2, Qt.SolidLine)

        qp.setPen(pen)
        points = [QPoint(20, 40), QPoint(250, 40), QPoint(250, 80), QPoint(20, 80)]
        qp.drawLines(points)


if __name__ == '__main__':

    import sys

    app = QApplication(sys.argv)
    ex = Example()
    sys.exit(app.exec_())
